@@ -14,8 +14,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 //BEGIN Includes
@@ -29,8 +29,6 @@
 #include "kateconfigplugindialogpage.h"
 #include "kateviewmanager.h"
 #include "kateapp.h"
-#include "kateprojectlist.h"
-#include "kateprojectviews.h"
 #include "katefileselector.h"
 #include "katefilelist.h"
 #include "kategrepdialog.h"
@@ -39,14 +37,16 @@
 #include "kateexternaltools.h"
 #include "katesavemodifieddialog.h"
 #include "katemwmodonhddialog.h"
+#include "katesession.h"
+#include "katetabwidget.h"
 
-#include <kmdi/tabwidget.h>
+#include "../interfaces/mainwindow.h"
+#include "../interfaces/toolviewmanager.h"
 
 #include <dcopclient.h>
 #include <kinstance.h>
 #include <kaboutdata.h>
 #include <kaction.h>
-#include <kapplication.h>
 #include <kcmdlineargs.h>
 #include <kdebug.h>
 #include <kdialogbase.h>
@@ -86,42 +86,64 @@
 
 uint KateMainWindow::uniqueID = 1;
 
-KateMainWindow::KateMainWindow () :
-    KMDI::MainWindow (0,(QString("__KateMainWindow#%1").arg(uniqueID)).latin1())
+KateMainWindow::KateMainWindow (KConfig *sconfig, const QString &sgroup)
+  : KateMDI::MainWindow (0,(QString("__KateMainWindow#%1").arg(uniqueID)).latin1())
 {
-  setToolViewStyle(KMultiTabBar::KDEV3ICON);
-  // make the dockwidgets keep their size if possible
-  manager()->setSplitterKeepSize(true);
   // first the very important id
   myID = uniqueID;
   uniqueID++;
 
-  // init some vars
-  m_project = 0;
-  m_projectNumber = 0;
-
   m_modignore = false;
 
   console = 0;
   greptool = 0;
 
-  // now the config
-  KConfig *config = kapp->config();
-
-  // first init size while we are still invisible, avoid flicker
+  // here we go, set some usable default sizes
   if (!initialGeometrySet())
   {
-    config->setGroup ("Kate Main Window");
     int scnum = QApplication::desktop()->screenNumber(parentWidget());
     QRect desk = QApplication::desktop()->screenGeometry(scnum);
-    QSize s ( config->readNumEntry( QString::fromLatin1("Width %1").arg(desk.width()), 700 ),
-              config->readNumEntry( QString::fromLatin1("Height %1").arg(desk.height()), 480 ) );
 
-    resize (kMin (s.width(), desk.width()), kMin(s.height(), desk.height()));
+    QSize size;
+
+    // try to load size
+    if (sconfig)
+    {
+      sconfig->setGroup (sgroup);
+      size.setWidth (sconfig->readNumEntry( QString::fromLatin1("Width %1").arg(desk.width()), 0 ));
+      size.setHeight (sconfig->readNumEntry( QString::fromLatin1("Height %1").arg(desk.height()), 0 ));
+    }
+
+    // if thats fails, try to reuse size
+    if (size.isEmpty())
+    {
+      // first try to reuse size known from current or last created main window ;=)
+      if (KateApp::self()->mainWindows () > 0)
+      {
+        KateMainWindow *win = KateApp::self()->activeMainWindow ();
+
+        if (!win)
+          win = KateApp::self()->mainWindow (KateApp::self()->mainWindows ()-1);
+
+        size = win->size();
+      }
+      else // now fallback to hard defaults ;)
+      {
+        // first try global app config
+        KateApp::self()->config()->setGroup ("MainWindow");
+        size.setWidth (KateApp::self()->config()->readNumEntry( QString::fromLatin1("Width %1").arg(desk.width()), 0 ));
+        size.setHeight (KateApp::self()->config()->readNumEntry( QString::fromLatin1("Height %1").arg(desk.height()), 0 ));
+
+        if (size.isEmpty())
+          size = QSize (kMin (700, desk.width()), kMin(480, desk.height()));
+      }
+
+      resize (size);
+    }
   }
 
-  // apply settings
-  applyMainWindowSettings(kapp->config(), "Kate Main Window");
+  // start session restore if needed
+  startRestore (sconfig, sgroup);
 
   m_mainWindow = new Kate::MainWindow (this);
   m_toolViewManager = new Kate::ToolViewManager (this);
@@ -140,93 +162,83 @@ KateMainWindow::KateMainWindow () :
 
   KatePluginManager::self()->enableAllPluginsGUI (this);
 
-  // connect settings menu aboutToshow
+  if ( KateApp::self()->authorize("shell_access") )
+    Kate::Document::registerCommand(KateExternalToolsCommand::self());
+
+  // connect documents menu aboutToshow
   documentMenu = (QPopupMenu*)factory()->container("documents", this);
   connect(documentMenu, SIGNAL(aboutToShow()), this, SLOT(documentMenuAboutToShow()));
 
-  connect(KateProjectManager::self()->projectManager(),SIGNAL(projectDeleted(uint)),this,SLOT(projectDeleted(uint)));
-
   // caption update
   for (uint i = 0; i < KateDocManager::self()->documents(); i++)
     slotDocumentCreated (KateDocManager::self()->document(i));
 
   connect(KateDocManager::self(),SIGNAL(documentCreated(Kate::Document *)),this,SLOT(slotDocumentCreated(Kate::Document *)));
 
-  readOptions(config);
+  readOptions();
 
-  if (console)
-    console->loadConsoleIfNeeded();
+  if (sconfig)
+    m_viewManager->restoreViewConfiguration (sconfig, sgroup);
+
+  finishRestore ();
 
   setAcceptDrops(true);
-
-  // activate the first restored project, if any
-  if (KateProjectManager::self()->projects() > 0)
-    activateProject(KateProjectManager::self()->project(0));
-  else
-    activateProject(0);
 }
 
 KateMainWindow::~KateMainWindow()
 {
-  saveOptions(kapp->config());
+  // first, save our fallback window size ;)
+  KateApp::self()->config()->setGroup ("MainWindow");
+  saveWindowSize (KateApp::self()->config());
 
-  ((KateApp *)kapp)->removeMainWindow (this);
+  // save other options ;=)
+  saveOptions();
+
+  KateApp::self()->removeMainWindow (this);
 
   KatePluginManager::self()->disableAllPluginsGUI (this);
 
   delete m_dcop;
-  delete kscript;
 }
 
 void KateMainWindow::setupMainWindow ()
 {
-  greptool = new GrepTool( this, "greptool" );
-  greptool->installEventFilter( this );
-  connect(greptool, SIGNAL(itemSelected(const QString &,int)), this, SLOT(slotGrepToolItemSelected(const QString &,int)));
-  // WARNING HACK - anders: showing the greptool seems to make the menu accels work
-  greptool->show();
-  greptool->hide();
+  setToolViewStyle( KMultiTabBar::KDEV3ICON );
+
+  m_tabWidget = new KateTabWidget (centralWidget());
 
   m_viewManager = new KateViewManager (this);
 
-  filelist = new KateFileList (this, m_viewManager, this/*filelistDock*/, "filelist");
-  filelist->readConfig(kapp->config(), "Filelist");
-  addToolView(KDockWidget::DockLeft,filelist,SmallIcon("kmultiple"), i18n("Documents"));
+  KateMDI::ToolView *ft = createToolView("kate_filelist", KMultiTabBar::Left, SmallIcon("kmultiple"), i18n("Documents"));
+  filelist = new KateFileList (this, m_viewManager, ft, "filelist");
+  filelist->readConfig(KateApp::self()->config(), "Filelist");
 
-  QVBox *prBox = new QVBox (this,"projects");
-  addToolView(KDockWidget::DockLeft,prBox,SmallIcon("view_tree"), i18n("Projects"));
-  projectlist = new KateProjectList (this, prBox/*filelistDock*/, "projectlist");
-  projectviews = new KateProjectViews (this, prBox/*filelistDock*/, "projectviews");
-  prBox->setStretchFactor(projectviews, 2);
-  prBox->show ();
-  projectlist->show ();
-  projectviews->show ();
+  KateMDI::ToolView *t = createToolView("kate_fileselector", KMultiTabBar::Left, SmallIcon("fileopen"), i18n("Filesystem Browser"));
+  fileselector = new KateFileSelector( this, m_viewManager, t, "operator");
+  connect(fileselector->dirOperator(),SIGNAL(fileSelected(const KFileItem*)),this,SLOT(fileSelected(const KFileItem*)));
 
-  fileselector = new KateFileSelector( this, m_viewManager, /*fileselectorDock*/ this, "operator");
-  addToolView(KDockWidget::DockLeft,fileselector, SmallIcon("fileopen"), i18n("Filesystem Browser"));
-
-  // TEST
-  addToolView( KDockWidget::DockBottom, greptool, SmallIcon("filefind"), i18n("Find in Files") );
-  if (kapp->authorize("shell_access"))
+  // ONLY ALLOW SHELL ACCESS IF ALLOWED ;)
+  if (KateApp::self()->authorize("shell_access"))
   {
-     console = new KateConsole (this, "console",viewManager());
-     console->installEventFilter( this );
-     addToolView(KDockWidget::DockBottom,console, SmallIcon("konsole"), i18n("Terminal"));
+    t = createToolView("kate_greptool", KMultiTabBar::Bottom, SmallIcon("filefind"), i18n("Find in Files") );
+    greptool = new GrepTool( t, "greptool" );
+    connect(greptool, SIGNAL(itemSelected(const QString &,int)), this, SLOT(slotGrepToolItemSelected(const QString &,int)));
+    connect(t,SIGNAL(visibleChanged(bool)),this, SLOT(updateGrepDir (bool)));
+    // WARNING HACK - anders: showing the greptool seems to make the menu accels work
+    greptool->show();
+
+    t = createToolView("kate_console", KMultiTabBar::Bottom, SmallIcon("konsole"), i18n("Terminal"));
+    console = new KateConsole (this, t);
   }
 
-  connect(fileselector->dirOperator(),SIGNAL(fileSelected(const KFileItem*)),this,SLOT(fileSelected(const KFileItem*)));
+  // make per default the filelist visible, if we are in session restore, katemdi will skip this ;)
+  showToolView (ft);
 }
 
 void KateMainWindow::setupActions()
 {
   KAction *a;
 
-  kscript = new KScriptManager(this, "scriptmanager");
-  scriptMenu = new KActionMenu( i18n("&KDE Scripts"), actionCollection(), "scripts");
-  scriptMenu->setWhatsThis(i18n("This shows all available scripts and allows them to be executed."));
-  setupScripts();
-  connect( scriptMenu->popupMenu(), SIGNAL(activated( int)), this, SLOT(runScript( int )) );
-
   KStdAction::openNew( m_viewManager, SLOT( slotDocumentNew() ), actionCollection(), "file_new" )->setWhatsThis(i18n("Create a new document"));
   KStdAction::open( m_viewManager, SLOT( slotDocumentOpen() ), actionCollection(), "file_open" )->setWhatsThis(i18n("Open an existing document for editing"));
 
@@ -248,10 +260,13 @@ void KateMainWindow::setupActions()
   a=new KAction(i18n("&New Window"), "window_new", 0, this, SLOT(newWindow()), actionCollection(), "view_new_view");
   a->setWhatsThis(i18n("Create a new Kate view (a new window with the same document list)."));
 
-  externalTools = new KateExternalToolsMenuAction( i18n("External Tools"), actionCollection(), "tools_external", this );
-  externalTools->setWhatsThis( i18n("Launch external helper applications") );
+  if ( KateApp::self()->authorize("shell_access") )
+  {
+    externalTools = new KateExternalToolsMenuAction( i18n("External Tools"), actionCollection(), "tools_external", this );
+    externalTools->setWhatsThis( i18n("Launch external helper applications") );
+  }
 
-  showFullScreenAction = KStdAction::fullScreen( 0, 0, actionCollection(),this);
+  KToggleAction* showFullScreenAction = KStdAction::fullScreen( 0, 0, actionCollection(),this);
   connect( showFullScreenAction,SIGNAL(toggled(bool)), this,SLOT(slotFullScreen(bool)));
 
   documentOpenWith = new KActionMenu(i18n("Open W&ith"), actionCollection(), "file_open_with");
@@ -262,23 +277,15 @@ void KateMainWindow::setupActions()
   a=KStdAction::keyBindings(this, SLOT(editKeys()), actionCollection());
   a->setWhatsThis(i18n("Configure the application's keyboard shortcut assignments."));
 
-  a=KStdAction::configureToolbars(this, SLOT(slotEditToolbars()), actionCollection(), "set_configure_toolbars");
+  a=KStdAction::configureToolbars(this, SLOT(slotEditToolbars()), actionCollection());
   a->setWhatsThis(i18n("Configure which items should appear in the toolbar(s)."));
 
-  // project menu
-  a = new KAction(i18n("&New Project..."), "filenew", 0, this, SLOT(slotProjectNew()), actionCollection(), "project_new");
-  a = new KAction(i18n("&Open Project..."), "fileopen", 0, this, SLOT(slotProjectOpen()), actionCollection(), "project_open");
-  saveProject = new KAction(i18n("&Save Project"), "filesave", 0, this, SLOT(slotProjectSave()), actionCollection(), "project_save");
-  closeProject = new KAction(i18n("&Close Project"), "fileclose", 0, this, SLOT(slotProjectClose()), actionCollection(), "project_close");
-
-  recentProjects = new KRecentFilesAction (i18n("Open &Recent"), KShortcut(), this, SLOT(openConstURLProject (const KURL&)),actionCollection(), "project_open_recent");
-
-  settingsConfigure = KStdAction::preferences(this, SLOT(slotConfigure()), actionCollection(), "settings_configure");
+  KAction* settingsConfigure = KStdAction::preferences(this, SLOT(slotConfigure()), actionCollection(), "settings_configure");
   settingsConfigure->setWhatsThis(i18n("Configure various aspects of this application and the editing component."));
 
   // pipe to terminal action
-  if (kapp->authorize("shell_access"))
-    new KAction(i18n("&Pipe to Console"), "pipe", 0, this, SLOT(slotPipeToConsole()), actionCollection(), "tools_pipe_to_terminal");
+  if (KateApp::self()->authorize("shell_access"))
+    new KAction(i18n("&Pipe to Console"), "pipe", 0, console, SLOT(slotPipeToConsole()), actionCollection(), "tools_pipe_to_terminal");
 
   // tip of the day :-)
   KStdAction::tipOfDay( this, SLOT( tipOfTheDay() ), actionCollection() )->setWhatsThis(i18n("This shows useful tips on the use of this application."));
@@ -294,7 +301,20 @@ void KateMainWindow::setupActions()
 
   slotWindowActivated ();
 
-  projectlist->setupActions();
+  // session actions
+  new KAction(i18n("Menu entry Session->New", "&New"), "filenew", 0, KateSessionManager::self(), SLOT(sessionNew()), actionCollection(), "sessions_new");
+  new KAction(i18n("&Open..."), "fileopen", 0, KateSessionManager::self(), SLOT(sessionOpen()), actionCollection(), "sessions_open");
+  new KAction(i18n("&Save"), "filesave", 0, KateSessionManager::self(), SLOT(sessionSave()), actionCollection(), "sessions_save");
+  new KAction(i18n("Save &As..."), "filesaveas", 0, KateSessionManager::self(), SLOT(sessionSaveAs()), actionCollection(), "sessions_save_as");
+  new KAction(i18n("&Manage..."), "view_choose", 0, KateSessionManager::self(), SLOT(sessionManage()), actionCollection(), "sessions_manage");
+
+  // quick open menu ;)
+  new KateSessionsAction (i18n("&Quick Open"), actionCollection(), "sessions_list");
+}
+
+KateTabWidget *KateMainWindow::tabWidget ()
+{
+  return m_tabWidget;
 }
 
 void KateMainWindow::slotDocumentCloseAll() {
@@ -330,39 +350,26 @@ bool KateMainWindow::queryClose_internal() {
  */
 bool KateMainWindow::queryClose()
 {
-  // session saving, can we close all projects & views ?
+  // session saving, can we close all views ?
   // just test, not close them actually
-  if (kapp->sessionSaving())
+  if (KateApp::self()->sessionSaving())
   {
-    return ( KateProjectManager::self()->queryCloseAll () &&
-             queryClose_internal() );
+    return queryClose_internal ();
   }
 
   // normal closing of window
   // allow to close all windows until the last without restrictions
-  if ( ((KateApp *)kapp)->mainWindows () > 1 )
+  if ( KateApp::self()->mainWindows () > 1 )
     return true;
 
-  // last one: check if we can close all projects/document, try run
-  // and save projects/docs if we really close down !
-
-
-  if ( KateProjectManager::self()->queryCloseAll () &&
-       queryClose_internal() )
+  // last one: check if we can close all documents, try run
+  // and save docs if we really close down !
+  if ( queryClose_internal () )
   {
-    KConfig scfg("katesessionrc", false);
+    KateApp::self()->sessionManager()->saveActiveSession(true, true);
 
-    KConfig *config = kapp->config();
-    config->setGroup("General");
-
-    if (config->readBoolEntry("Restore Projects", false))
-      KateProjectManager::self()->saveProjectList (&scfg);
-
-    if (config->readBoolEntry("Restore Documents", false))
-      KateDocManager::self()->saveDocumentList (&scfg);
-
-    if (config->readBoolEntry("Restore Window Configuration", false))
-      saveProperties (&scfg);
+    // detach the dcopClient
+    KateApp::self()->dcopClient()->detach();
 
     return true;
   }
@@ -372,23 +379,31 @@ bool KateMainWindow::queryClose()
 
 void KateMainWindow::newWindow ()
 {
-  ((KateApp *)kapp)->newMainWindow ();
+  KateApp::self()->newMainWindow ();
 }
 
 void KateMainWindow::slotEditToolbars()
 {
+  saveMainWindowSettings( KateApp::self()->config(), "MainWindow" );
   KEditToolbar dlg( factory() );
-
+  connect( &dlg, SIGNAL(newToolbarConfig()), this, SLOT(slotNewToolbarConfig()) );
   dlg.exec();
 }
 
+void KateMainWindow::slotNewToolbarConfig()
+{
+  applyMainWindowSettings( KateApp::self()->config(), "MainWindow" );
+}
+
 void KateMainWindow::slotFileQuit()
 {
-  close ();
+  KateApp::self()->shutdownKate (this);
 }
 
-void KateMainWindow::readOptions(KConfig *config)
+void KateMainWindow::readOptions ()
 {
+  KConfig *config = KateApp::self()->config ();
+
   config->setGroup("General");
   syncKonsole =  config->readBoolEntry("Sync Konsole", true);
   modNotification = config->readBoolEntry("Modified Notification", false);
@@ -400,13 +415,11 @@ void KateMainWindow::readOptions(KConfig *config)
   fileOpenRecent->loadEntries(config, "Recent Files");
 
   fileselector->readConfig(config, "fileselector");
-
-  recentProjects->loadEntries (config, "Recent Projects");
 }
 
-void KateMainWindow::saveOptions(KConfig *config)
+void KateMainWindow::saveOptions ()
 {
-  saveMainWindowSettings(config, "Kate Main Window");
+  KConfig *config = KateApp::self()->config ();
 
   config->setGroup("General");
 
@@ -428,18 +441,15 @@ void KateMainWindow::saveOptions(KConfig *config)
   fileselector->writeConfig(config, "fileselector");
 
   filelist->writeConfig(config, "Filelist");
-
-  recentProjects->saveEntries (config, "Recent Projects");
 }
 
 void KateMainWindow::slotWindowActivated ()
 {
-  static QString path;
-
   if (m_viewManager->activeView())
   {
     if (console && syncKonsole)
     {
+      static QString path;
       QString newPath = m_viewManager->activeView()->getDoc()->url().directory();
 
       if ( newPath != path )
@@ -451,6 +461,9 @@ void KateMainWindow::slotWindowActivated ()
 
     updateCaption (m_viewManager->activeView()->getDoc());
   }
+
+  // update proxy
+  centralWidget()->setFocusProxy (m_viewManager->activeView());
 }
 
 void KateMainWindow::slotUpdateOpenWith()
@@ -463,14 +476,16 @@ void KateMainWindow::slotUpdateOpenWith()
 
 void KateMainWindow::documentMenuAboutToShow()
 {
-  documentMenu->clear ();
+  // remove documents
+  while (documentMenu->count() > 3)
+    documentMenu->removeItemAt (3);
 
   QListViewItem * item = filelist->firstChild();
   while( item ) {
     documentMenu->insertItem (
-	  KStringHandler::rsqueeze( ((KateFileListItem *)item)->document()->docName(), 150 ), // would it be saner to use the screen width as a limit that some random number??
+          KStringHandler::rsqueeze( ((KateFileListItem *)item)->document()->docName(), 150 ), // would it be saner to use the screen width as a limit that some random number??
           m_viewManager, SLOT (activateView (int)), 0,
-	  ((KateFileListItem *)item)->documentNumber () );
+          ((KateFileListItem *)item)->documentNumber () );
 
     item = item->nextSibling();
   }
@@ -565,17 +580,13 @@ KURL KateMainWindow::activeDocumentUrl()
 
 void KateMainWindow::fileSelected(const KFileItem * /*file*/)
 {
-  /*if (file) {
-  	m_viewManager->openURL( file->url() );
- } else */{
- 	const KFileItemList *list=fileselector->dirOperator()->selectedItems();
-	KFileItem *tmp;
-	for (KFileItemListIterator it(*list); (tmp = it.current()); ++it) {
-		m_viewManager->openURL(tmp->url());
-		fileselector->dirOperator()->view()->setSelected(tmp,false);
-	}
- }
- 	//fileSelector->dirOperator()->
+  const KFileItemList *list=fileselector->dirOperator()->selectedItems();
+  KFileItem *tmp;
+  for (KFileItemListIterator it(*list); (tmp = it.current()); ++it)
+  {
+    m_viewManager->openURL(tmp->url());
+    fileselector->dirOperator()->view()->setSelected(tmp,false);
+  }
 }
 
 // TODO make this work
@@ -602,43 +613,33 @@ void KateMainWindow::slotOpenWithMenuAction(int idx)
   KURL::List list;
   list.append( m_viewManager->activeView()->getDoc()->url() );
   QString* appname = new QString( documentOpenWith->popupMenu()->text(idx) );
-  if ( appname->compare(i18n("&Other...")) == 0 ) {
+
+  *appname = appname->remove('&'); //Remove a possible accelerator ... otherwise the application might not get found.
+  if ( appname->compare(i18n("Other...")) == 0 ) {
     // display "open with" dialog
     KOpenWithDlg* dlg = new KOpenWithDlg(list);
     if (dlg->exec())
       KRun::run(*dlg->service(), list);
     return;
   }
+
   QString qry = QString("((Type == 'Application') and (Name == '%1'))").arg( appname->latin1() );
   KMimeType::Ptr mime = KMimeType::findByURL( m_viewManager->activeView()->getDoc()->url() );
   KTrader::OfferList offers = KTrader::self()->query(mime->name(), qry);
-  KService::Ptr app = offers.first();
-  // some checking here: pop a wacko message it the app wasn't found.
-  KRun::run(*app, list);
+
+  if (!offers.isEmpty()) {
+    KService::Ptr app = offers.first();
+    KRun::run(*app, list);
+  }
+  else
+    KMessageBox::error(this, i18n("Application '%1' not found!").arg(appname->latin1()), i18n("Application Not Found!"));
+
+
 }
 
 void KateMainWindow::pluginHelp()
 {
-  kapp->invokeHelp (QString::null, "kate-plugins");
-}
-
-void KateMainWindow::setupScripts()
-{
-  // locate all scripts, local as well as global.
-  // The script manager will do the nessecary sanity checking
-  QStringList scripts = KGlobal::dirs()->findAllResources("data", QString(kapp->name())+"/scripts/*.desktop", false, true );
-  for (QStringList::Iterator it = scripts.begin(); it != scripts.end(); ++it )
-    kscript->addScript( *it );
-  QStringList l ( kscript->scripts() );
-  for (QStringList::Iterator it=l.begin(); it != l.end(); ++it )
-    scriptMenu->popupMenu()->insertItem( *it );
-}
-
-void KateMainWindow::runScript( int mIId )
-{
-	//kdDebug(13001) << "Starting script engine..." << endl;
-//         kdDebug(13001)<<"runScript( "<<mIId<<" ) ["<<scriptMenu->popupMenu()->text( mIId )<<"]"<<endl;
-	kscript->runScript( scriptMenu->popupMenu()->text( mIId ) );
+  KateApp::self()->invokeHelp (QString::null, "kate-plugins");
 }
 
 void KateMainWindow::slotMail()
@@ -661,7 +662,7 @@ void KateMainWindow::slotMail()
               i18n("<p>The current document has not been saved, and "
               "cannot be attached to an email message."
               "<p>Do you want to save it and proceed?"),
-              i18n("Cannot Send Unsaved File") );
+              i18n("Cannot Send Unsaved File"),KStdGuiItem::saveAs(),KStdGuiItem::cancel() );
       if ( r == KMessageBox::Yes ) {
         Kate::View *v = (Kate::View*)doc->views().first();
         int sr = v->saveAs();
@@ -683,7 +684,7 @@ void KateMainWindow::slotMail()
                 i18n("<p>The current file:<br><strong>%1</strong><br>has been "
                 "modified. Modifications will not be available in the attachment."
                 "<p>Do you want to save it before sending it?").arg(doc->url().prettyURL()),
-                i18n("Save Before Sending?") );
+                i18n("Save Before Sending?"), KStdGuiItem::save(), i18n("Do Not Save") );
       switch ( r ) {
         case KMessageBox::Cancel:
           continue;
@@ -704,7 +705,7 @@ void KateMainWindow::slotMail()
   } // check selected docs done
   if ( ! urls.count() )
     return;
-  kapp->invokeMailer( QString::null, // to
+  KateApp::self()->invokeMailer( QString::null, // to
                       QString::null, // cc
                       QString::null, // bcc
                       QString::null, // subject
@@ -726,25 +727,19 @@ void KateMainWindow::slotFullScreen(bool t)
     showNormal();
 }
 
-bool KateMainWindow::eventFilter( QObject *o, QEvent *e )
+void KateMainWindow::updateGrepDir (bool visible)
 {
+  // grepdlg gets hidden
+  if (!visible)
+    return;
 
-  if ( o == greptool && e->type() == QEvent::Show && m_viewManager->activeView() )
+  if ( m_viewManager->activeView() )
   {
     if ( m_viewManager->activeView()->getDoc()->url().isLocalFile() )
     {
       greptool->updateDirName( m_viewManager->activeView()->getDoc()->url().directory() );
-      return true;
     }
   }
-  if ( ( o == greptool || o == console ) &&
-      e->type() == QEvent::Hide && m_viewManager->activeView() )
-  {
-     m_viewManager->activeView()->setFocus();
-     return true;
-  }
-
-  return KMDI::MainWindow::eventFilter( o, e );
 }
 
 bool KateMainWindow::event( QEvent *e )
@@ -755,7 +750,7 @@ bool KateMainWindow::event( QEvent *e )
     if ( m_modignore )
     {
       m_modignore = false;
-      return KMDI::MainWindow::event( e );
+      return KateMDI::MainWindow::event( e );
     }
     showModOnDiskPrompt();
   }
@@ -764,7 +759,7 @@ bool KateMainWindow::event( QEvent *e )
   else if ( (type == QEvent::WindowUnblocked || type == QEvent::WindowBlocked) && modNotification)
     m_modignore = true;
 
-  return KMDI::MainWindow::event( e );
+  return KateMDI::MainWindow::event( e );
 }
 
 bool KateMainWindow::showModOnDiskPrompt()
@@ -793,124 +788,6 @@ bool KateMainWindow::showModOnDiskPrompt()
   return true;
 }
 
-KMDI::ToolViewAccessor *KateMainWindow::addToolView(KDockWidget::DockPosition position, QWidget *widget, const QPixmap &icon, const QString &sname, const QString &tabToolTip, const QString &tabCaption)
-{
-  widget->setIcon(icon);
-  widget->setCaption(sname);
-
-  return addToolWindow(widget, position, getMainDockWidget(), 25, tabToolTip, tabCaption);
-}
-
-bool KateMainWindow::removeToolView(QWidget *w)
-{
-  deleteToolWindow (w);
-  return true;
-}
-
-bool KateMainWindow::removeToolView(KMDI::ToolViewAccessor *accessor)
-{
-  deleteToolWindow (accessor);
-  return true;
-}
-
-bool KateMainWindow::showToolView(QWidget *){return false;}
-bool KateMainWindow::showToolView(KMDI::ToolViewAccessor *){return false;}
-
-bool KateMainWindow::hideToolView(QWidget *){return false;}
-bool KateMainWindow::hideToolView(KMDI::ToolViewAccessor *){return false;}
-
-void KateMainWindow::slotProjectNew ()
-{
-  ProjectInfo *info = KateProjectManager::self()->newProjectDialog (this);
-
-  if (info)
-  {
-    createProject (info->type, info->name, info->fileName);
-    delete info;
-  }
-}
-
-void KateMainWindow::slotProjectOpen ()
-{
-  QString fileName = KFileDialog::getOpenFileName (QString::null, QString ("*.kateproject|") + i18n("Kate Project Files") + QString (" (*.kateproject)"), this, i18n("Open Kate Project"));
-
-  if (!fileName.isEmpty())
-    openProject (fileName);
-}
-
-void KateMainWindow::slotProjectSave ()
-{
-  if (m_project)
-    m_project->save ();
-}
-
-void KateMainWindow::slotProjectClose ()
-{
-  if (m_project)
-  {
-    KateProjectManager::self()->close (m_project);
-  }
-}
-
-void KateMainWindow::activateProject (Kate::Project *project)
-{
-//   kdDebug(13001)<<"activating project "<<project<<endl;
-  if (m_project)
-    KateProjectManager::self()->disableProjectGUI (m_project, this);
-
-  if (project)
-    KateProjectManager::self()->enableProjectGUI (project, this);
-
-  m_project = project;
-
-  if (project)
-  {
-    KateProjectManager::self()->setCurrentProject (project);
-    m_projectNumber = project->projectNumber ();
-  }
-  else
-    m_projectNumber = 0;
-
-  saveProject->setEnabled(project != 0);
-  closeProject->setEnabled(project != 0);
-
-  emit m_mainWindow->projectChanged ();
-}
-
-Kate::Project *KateMainWindow::createProject (const QString &type, const QString &name, const QString &filename)
-{
-  Kate::Project *project = KateProjectManager::self()->create (type, name, filename);
-
-  if (project)
-    activateProject (project);
-
-  return project;
-}
-
-Kate::Project *KateMainWindow::openProject (const QString &filename)
-{
-  Kate::Project *project = KateProjectManager::self()->open (filename);
-
-  if (project)
-  {
-    recentProjects->addURL ( KURL(filename) );
-    activateProject (project);
-  }
-
-  return project;
-}
-
-void KateMainWindow::projectDeleted (uint projectNumber)
-{
-  if (projectNumber == m_projectNumber)
-  {
-    if (KateProjectManager::self()->projects() > 0)
-      activateProject (KateProjectManager::self()->project(KateProjectManager::self()->projects()-1));
-    else
-      activateProject (0);
-  }
-}
-
 void KateMainWindow::slotDocumentCreated (Kate::Document *doc)
 {
   connect(doc,SIGNAL(modStateChanged(Kate::Document *)),this,SLOT(updateCaption(Kate::Document *)));
@@ -941,80 +818,41 @@ void KateMainWindow::updateCaption (Kate::Document *doc)
     c = m_viewManager->activeView()->getDoc()->url().prettyURL();
   }
 
-  setCaption( KStringHandler::lsqueeze(c,64), m_viewManager->activeView()->getDoc()->isModified());
+  QString sessName = KateApp::self()->sessionManager()->activeSession()->sessionName();
+  if ( !sessName.isEmpty() )
+    sessName = QString("%1: ").arg( sessName );
+
+  setCaption( sessName + KStringHandler::lsqueeze(c,64),
+      m_viewManager->activeView()->getDoc()->isModified());
 }
 
-void KateMainWindow::openConstURLProject (const KURL&url)
+void KateMainWindow::saveProperties(KConfig *config)
 {
-  openProject (url.path());
-}
-
-void KateMainWindow::saveProperties(KConfig *config) {
-  kdDebug(13000)<<"KateMainWindow::saveProperties(): group: "<<config->group()<<endl;
-  assert(config);
-
-//   kdDebug(13000)<<"preparing session saving"<<endl;
   QString grp=config->group();
-  QString dockGrp;
 
-  if (kapp->sessionSaving()) dockGrp=grp+"-Docking";
-	else dockGrp="MainWindow0-Docking";
-/*  if (config->readNumEntry("GUIMode",KMdi::UndefinedMode)!=mdiMode()) {
-        config->writeEntry("GUIMode",mdiMode());
-        config->deleteGroup("MainWindow0-Docking");
-  }*/
+  saveSession(config, grp);
+  m_viewManager->saveViewConfiguration (config, grp);
 
-//   kdDebug(13000)<<"Before write dock config"<<endl;
-  writeDockConfig(config,dockGrp);
-//   kdDebug(13000)<<"After write dock config"<<endl;
-
-
-  if (kapp->sessionSaving()) dockGrp=grp+"-View Configuration";
-	else dockGrp="MainWindow0-View Configuration";
-
-  m_viewManager->saveViewConfiguration (config,dockGrp);
-//   kdDebug(13000)<<"After saving view configuration"<<endl;
   config->setGroup(grp);
-
 }
 
 void KateMainWindow::readProperties(KConfig *config)
 {
   QString grp=config->group();
-  QString dockGrp;
 
-  if (kapp->isRestored()) dockGrp=grp+"-Docking";
-	else dockGrp="MainWindow0-Docking";
+  startRestore(config, grp);
+  finishRestore ();
+  m_viewManager->restoreViewConfiguration (config, grp);
 
-  if (config->hasGroup(dockGrp))
-        readDockConfig(config,dockGrp);
-
-  if (kapp->isRestored()) dockGrp=grp+"-View Configuration";
-	else dockGrp="MainWindow0-View Configuration";
-
-  m_viewManager->restoreViewConfiguration (config,dockGrp);
   config->setGroup(grp);
 }
 
 void KateMainWindow::saveGlobalProperties( KConfig* sessionConfig )
 {
-  KateProjectManager::self()->saveProjectList (sessionConfig);
   KateDocManager::self()->saveDocumentList (sessionConfig);
+
+  sessionConfig->setGroup("General");
+  sessionConfig->writeEntry ("Last Session", KateApp::self()->sessionManager()->activeSession()->sessionFileRelative());
 }
 
-void KateMainWindow::slotPipeToConsole ()
-{
-  if (!console)
-    return;
-
-  Kate::View *v = m_viewManager->activeView();
-
-  if (!v)
-    return;
-
-  if (v->getDoc()->hasSelection ())
-    console->sendInput (v->getDoc()->selection());
-  else
-    console->sendInput (v->getDoc()->text());
-}
 // kate: space-indent on; indent-width 2; replace-tabs on;
