@@ -95,7 +95,9 @@ void MenuTab::load()
                                           c->readBoolEntry("UseBrowser", false));
     connect(m_quickBrowserMenu, SIGNAL(toggled(bool)), SIGNAL(changed()));
 
-    QStringList ext = c->readListEntry("Extensions", "prefmenu.desktop,systemmenu.desktop");
+    QStringList ext_default;
+    ext_default << "prefmenu.desktop" << "systemmenu.desktop";
+    QStringList ext = c->readListEntry("Extensions", ext_default);
     QStringList dirs = KGlobal::dirs()->findDirs("data", "kicker/menuext");
     kSubMenuItem* menuItem(0);
     for (QStringList::ConstIterator dit=dirs.begin(); dit!=dirs.end(); ++dit)
