@@ -98,14 +98,14 @@ LanBrowser::LanBrowser(QWidget *parent)
 		"can be found at <a href=\"http://lisa-home.sourceforge.net\">"
 		"the LISa Homepage</a> or contact Alexander Neundorf "
 		"&lt;<a href=\"mailto:neundorf@kde.org\">neundorf@kde.org</a>&gt;."));
-   
+
    layout.addWidget(&tabs);
 
    smbPage = create_smb(&tabs, 0);
    tabs.addTab(smbPage, i18n("&Windows Shares"));
    connect(smbPage,SIGNAL(changed(bool)), SLOT( changed() ));
 
-   lisaPage = KCModuleLoader::loadModule("kcmlisa", &tabs);
+   lisaPage = KCModuleLoader::loadModule("kcmlisa", KCModuleLoader::Both, &tabs);
    if (lisaPage)
    {
      tabs.addTab(lisaPage,i18n("&LISa Daemon"));
@@ -119,7 +119,7 @@ LanBrowser::LanBrowser(QWidget *parent)
 //     connect(resLisaPage,SIGNAL(changed()), SLOT( changed() ));
 //   }
 
-   kioLanPage = KCModuleLoader::loadModule("kcmkiolan", &tabs);
+   kioLanPage = KCModuleLoader::loadModule("kcmkiolan",  KCModuleLoader::Both, &tabs);
    if (kioLanPage)
    {
      tabs.addTab(kioLanPage,i18n("lan:/ Iosla&ve"));
