@@ -15,29 +15,9 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
-#include <config.h>
-
-#include "nic.h"
-
-#include <kdialog.h>
-#include <kgenericfactory.h>
-#include <kaboutdata.h>
-
-
-#include <qtimer.h>
-#include <qtabwidget.h>
-#include <kglobal.h>
-#include <qlayout.h>
-#include <qpushbutton.h>
-#include <qlistview.h>
-
-#ifdef HAVE_SYS_SOCKIO_H
-#include <sys/sockio.h>
-#endif
-
 #include <sys/types.h>
 #include <sys/param.h>
 #include <netinet/in.h>
@@ -45,6 +25,23 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <sys/socket.h>
+#include <config.h>
+#ifdef HAVE_SYS_SOCKIO_H
+#include <sys/sockio.h>
+#endif
+
+#include <kaboutdata.h>
+#include <kdialog.h>
+#include <kgenericfactory.h>
+#include <kglobal.h>
+
+#include <qlayout.h>
+#include <qlistview.h>
+#include <qpushbutton.h>
+#include <qtabwidget.h>
+#include <qtimer.h>
+
+#include "nic.h"
 
 #ifdef USE_SOLARIS
 /* net/if.h is incompatible with STL on Solaris 2.6 - 2.8, redefine
@@ -80,6 +77,7 @@ struct MyNIC
    QString netmask;
    QString state;
    QString type;
+   QString HWaddr;
 };
 
 typedef QPtrList<MyNIC> NICList;
@@ -97,6 +95,7 @@ KCMNic::KCMNic(QWidget *parent, const char * name, const QStringList &)
    m_list->addColumn(i18n("Network Mask"));
    m_list->addColumn(i18n("Type"));
    m_list->addColumn(i18n("State"));
+   m_list->addColumn(i18n("HWaddr"));
    m_list->setAllColumnsShowFocus(true);
    QHBoxLayout *hbox=new QHBoxLayout(box);
    m_updateButton=new QPushButton(i18n("&Update"),this);
@@ -124,10 +123,25 @@ void KCMNic::update()
    NICList *nics=findNICs();
    nics->setAutoDelete(true);
    for (MyNIC* tmp=nics->first(); tmp!=0; tmp=nics->next())
-      new QListViewItem(m_list,tmp->name, tmp->addr, tmp->netmask, tmp->type, tmp->state);
+      new QListViewItem(m_list,tmp->name, tmp->addr, tmp->netmask, tmp->type, tmp->state, tmp->HWaddr);
    delete nics;
 }
 
+static QString HWaddr2String( char *hwaddr )
+{
+   QString ret;
+   int i;
+   for (i=0; i<6; i++, hwaddr++) {
+	int v = (*hwaddr & 0xff);
+	QString num = QString("%1").arg(v,0,16);
+	if (num.length() < 2)
+		num.prepend("0");
+	if (i>0)
+		ret.append(":");
+	ret.append(num);
+   }
+   return ret;
+}
 
 NICList* findNICs()
 {
@@ -199,6 +213,17 @@ NICList* findNICs()
          }
          else
             tmp->netmask=i18n("Unknown");
+
+         ifcopy=*ifr;
+         result=ioctl(sockfd,SIOCGIFHWADDR,&ifcopy);
+         if (result==0)
+         {
+            char *n = &ifcopy.ifr_ifru.ifru_hwaddr.sa_data[0];
+            tmp->HWaddr = HWaddr2String(n);
+         }
+         else
+            tmp->HWaddr = i18n("Unknown");
+
          nl->append(tmp);
          break;
 
