@@ -45,6 +45,7 @@
 #include <qfile.h>
 #include <qsocketnotifier.h>
 #include <qvaluevector.h>
+#include <qtooltip.h>
 
 #include <qdatetime.h>
 
@@ -119,6 +120,10 @@ LockProcess::LockProcess(bool child, bool useBlankOnly)
                         qt_xscreen()), &rootAttr);
     mRootWidth = rootAttr.width;
     mRootHeight = rootAttr.height;
+    { // trigger creation of QToolTipManager, it does XSelectInput() on the root window
+    QWidget w;
+    QToolTip::add( &w, "foo" );
+    }
     XSelectInput( qt_xdisplay(), qt_xrootwin(),
         SubstructureNotifyMask | rootAttr.your_event_mask );
 
@@ -501,6 +506,7 @@ void LockProcess::saveVRoot()
       {
         gVRoot = children[i];
         gVRootData = *newRoot;
+        XFree ((char*) newRoot);
         break;
       }
     }
@@ -857,7 +863,19 @@ bool LockProcess::checkPass()
     killTimer(mAutoLogoutTimerId);
     PasswordDlg passDlg( this, &greetPlugin);
 
-    return execDialog( &passDlg ) == QDialog::Accepted;
+    int ret = execDialog( &passDlg );
+
+    XWindowAttributes rootAttr;
+    XGetWindowAttributes(qt_xdisplay(), RootWindow(qt_xdisplay(),
+                        qt_xscreen()), &rootAttr);
+    if(( rootAttr.your_event_mask & SubstructureNotifyMask ) == 0 )
+    {
+        kdWarning() << "ERROR: Something removed SubstructureNotifyMask from the root window!!!" << endl;
+        XSelectInput( qt_xdisplay(), qt_xrootwin(),
+            SubstructureNotifyMask | rootAttr.your_event_mask );
+    }
+
+    return ret == QDialog::Accepted;
 }
 
 static void fakeFocusIn( WId window )
