@@ -73,7 +73,7 @@
 #define KDESU_ERR strerror(errno)
 
 Minicli::Minicli( QWidget *parent, const char *name)
-        :KDialog( parent, name ),
+        :KDialog( parent, name, false, WType_TopLevel ),
          m_autoCheckedRunInTerm(false)
 {
   setPlainCaption( i18n("Run Command") );
@@ -474,6 +474,7 @@ int Minicli::runCommand()
     proc.setTerminal(true);
     proc.setErase(true);
     _exit(proc.exec(m_dlg->lePassword->password()));
+    return 0;
   }
   else
   {
@@ -503,7 +504,7 @@ int Minicli::runCommand()
         case KURIFilterData::HELP:
         {
           // No need for kfmclient, KRun does it all (David)
-          (void) new KRun( m_filterData->uri() );
+          (void) new KRun( m_filterData->uri(), parentWidget());
           return 0;
         }
         case KURIFilterData::EXECUTABLE:
@@ -549,7 +550,7 @@ int Minicli::runCommand()
           if (service && service->isValid() && service->type() == "Application")
           {
             notifyServiceStarted(service);
-            KRun::run(*service, KURL::List());
+            KRun::run(*service, KURL::List(), this);
             return 0;
           }
 
@@ -557,7 +558,7 @@ int Minicli::runCommand()
           if (service && service->isValid() && service->type() == "Application")
           {
             notifyServiceStarted(service);
-            KRun::run(*service, KURL::List());
+            KRun::run(*service, KURL::List(), this);
             return 0;
           }
 
