@@ -21,8 +21,8 @@
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program in a file called COPYING; if not, write to
-** the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-** MA 02111-1307, USA.
+** the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+** MA 02110-1301, USA.
 */
 
 /*
@@ -77,7 +77,7 @@
 
 KasGroupItem::KasGroupItem( KasTasker *parent )
     : KasItem( parent ), items(), groupType_( GroupRelated )
-
+     
 {
     setCustomPopup( true );
     setGroupItem( true );
@@ -119,7 +119,19 @@ void KasGroupItem::removeTask( Task::Ptr t )
 	return;
 
     hidePopup();
-    items.remove( t );
+
+    for (Task::List::iterator it = items.begin(); it != items.end();)
+    {
+        if ((*it) == t)
+        {
+            it = items.erase(it);
+        }
+        else
+        {
+            ++it;
+        }
+    }
+
     updateIcon();
 
     if ( items.count() == 1 )
@@ -158,8 +170,7 @@ void KasGroupItem::paint( QPainter *p )
     // Item summary info
     //
     int modCount = 0;
-	TaskList::iterator itEnd = items.end();
-	for (TaskList::iterator it = items.begin(); it != itEnd; ++it) {
+    for ( Task::List::iterator it = items.begin(); it != items.end() ; ++it ) {
 	if ( (*it)->isModified() )
 	    modCount++;
     }
@@ -202,9 +213,7 @@ void KasGroupItem::paint( QPainter *p )
     int ypos = 16;
 
     for ( int i = 0; ( i < (int) items.count() ) && ( i < microsPerCol ); i++ ) {
-	Task::Ptr t = *items.at( i );
-	if (!t)
-	    break;
+	Task::Ptr t = items.at( i );
 
 	if( t->isIconified() )
 	    p->drawPixmap( xpos, ypos, res->microMinIcon() );
@@ -232,9 +241,8 @@ void KasGroupItem::updatePopup()
 
 	bar->clear();
 	if ( items.count() ) {
-		TaskList::iterator itEnd = items.end();
-		for (TaskList::iterator it = items.begin(); it != itEnd; ++it) {
-			bar->addTask( *it );
+	    for ( Task::List::iterator t = items.begin(); t != items.end(); ++t ) {
+		bar->addTask( *t );
 	    }
 	}
 
@@ -257,7 +265,7 @@ KasPopup *KasGroupItem::createPopup()
 //     //
 //     // This generates cool looking fractal-like patterns if you keep unfolding the
 //     // groups!
-//     int pos = (int) this;
+//     int pos = (int) this; 
 //     if ( pos % 2 )
 //        bar->append( new KasItem( bar ) );
 //     if ( pos % 5 )
@@ -282,7 +290,7 @@ void KasGroupItem::showGroupMenuAt( QMouseEvent *ev )
 
 void KasGroupItem::showGroupMenuAt( const QPoint &p )
 {
-    TaskRMBMenu *tm = new TaskRMBMenu(&items, true, kasbar());
+    TaskRMBMenu *tm = new TaskRMBMenu(items, true, kasbar());
     tm->insertItem( i18n("&Ungroup" ), this, SLOT( ungroup() ) );
     tm->insertSeparator();
     tm->insertItem( i18n("&Kasbar"), kasbar()->contextMenu() );
