@@ -19,13 +19,17 @@ License. See the file "COPYING" for the exact licensing terms.
 #include "client.h"
 #include "workspace.h"
 
+#include <fixx11h.h>
 #include <qpopupmenu.h>
 #include <kxerrorhandler.h>
 #include <kstartupinfo.h>
+#include <kstringhandler.h>
+#include <klocale.h>
 
 #include "notifications.h"
 #include "atoms.h"
 #include "group.h"
+#include "rules.h"
 
 extern Time qt_x_time;
 
@@ -213,31 +217,31 @@ void Workspace::setActiveClient( Client* c, allowed_t )
     {
     if ( active_client == c )
         return;
-    if( popup && popup_client != c && set_active_client_recursion == 0 ) 
-        {
-        popup->close();
-        popup_client = 0;
-        }
+    if( active_popup && active_popup_client != c && set_active_client_recursion == 0 ) 
+        closeActivePopup();
     StackingUpdatesBlocker blocker( this );
     ++set_active_client_recursion;
     if( active_client != NULL )
         { // note that this may call setActiveClient( NULL ), therefore the recursion counter
-        active_client->setActive( false );
-        }
+            active_client->setActive( false, !c || !c->isModal() || c != active_client->transientFor() );
+    }
     active_client = c;
     Q_ASSERT( c == NULL || c->isActive());
     if( active_client != NULL )
         last_active_client = active_client;
     if ( active_client ) 
         {
-        focus_chain.remove( c );
-        if ( c->wantsTabFocus() )
-            focus_chain.append( c );
+        updateFocusChains( active_client, true ); // make it first in focus chain
         active_client->demandAttention( false );
         }
+    pending_take_activity = NULL;
 
     updateCurrentTopMenu();
     updateToolWindows( false );
+    if( c )
+        disableGlobalShortcutsForClient( c->rules()->checkDisableGlobalShortcuts( false ));
+    else
+        disableGlobalShortcutsForClient( false );
 
     updateStackingOrder(); // e.g. fullscreens have different layer when active/not-active
 
@@ -270,12 +274,12 @@ void Workspace::activateClient( Client* c, bool force )
         ++block_focus;
         setCurrentDesktop( c->desktop() );
         --block_focus;
-        // popupinfo->showInfo( desktopName(currentDesktop()) ); // AK - not sure
         }
     if( c->isMinimized())
         c->unminimize();
 
-    if( options->focusPolicyIsReasonable())
+// TODO force should perhaps allow this only if the window already contains the mouse
+    if( options->focusPolicyIsReasonable() || force )
         requestFocus( c, force );
 
     // Don't update user time for clients that have focus stealing workaround.
@@ -297,45 +301,74 @@ void Workspace::activateClient( Client* c, bool force )
   \sa Workspace::activateClient()
  */
 void Workspace::requestFocus( Client* c, bool force )
-    { // the 'if( c == active_client ) return;' optimization mustn't be done here
+    {
+    takeActivity( c, ActivityFocus | ( force ? ActivityFocusForce : 0 ), false);
+    }
+    
+void Workspace::takeActivity( Client* c, int flags, bool handled )
+    {
+     // the 'if( c == active_client ) return;' optimization mustn't be done here
     if (!focusChangeEnabled() && ( c != active_client) )
-        return;
+        flags &= ~ActivityFocus;
 
-    //TODO will be different for non-root clients. (subclassing?)
     if ( !c ) 
         {
         focusToNull();
         return;
         }
 
-    if( !c->isOnCurrentDesktop()) // shouldn't happen, call activateClient() if needed
+    if( flags & ActivityFocus )
         {
-        kdWarning( 1212 ) << "requestFocus: not on current desktop" << endl;
-        return;
+        Client* modal = c->findModal();
+        if( modal != NULL && modal != c )	
+            { 
+            if( !modal->isOnDesktop( c->desktop()))
+                {
+                modal->setDesktop( c->desktop());
+                if( modal->desktop() != c->desktop()) // forced desktop
+                    activateClient( modal );
+                }
+            // if the click was inside the window (i.e. handled is set),
+            // but it has a modal, there's no need to use handled mode, because
+            // the modal doesn't get the click anyway
+            // raising of the original window needs to be still done
+            if( flags & ActivityRaise )
+                raiseClient( c );
+            c = modal;
+            handled = false;
+            }
+        cancelDelayFocus();
         }
-
-    Client* modal = c->findModal();
-    if( modal != NULL && modal != c )	
-        { 
-        if( !modal->isOnDesktop( c->desktop())) // move the modal to client's desktop
-            modal->setDesktop( c->isOnAllDesktops() ? currentDesktop() : c->desktop());
-        requestFocus( modal, force );
-        return;
-        }
-    if ( c->isShown( false ) ) 
-        {
-        c->takeFocus( force, Allowed );
-        should_get_focus.append( c );
-        focus_chain.remove( c );
-        if ( c->wantsTabFocus() )
-            focus_chain.append( c );
-        }
-    else if ( c->isShade() && c->wantsInput()) 
+    if ( !( flags & ActivityFocusForce ) && ( c->isTopMenu() || c->isDock() || c->isSplash()) )
+        flags &= ~ActivityFocus; // toplevel menus and dock windows don't take focus if not forced
+    if( c->isShade())
         {
+        if( c->wantsInput() && ( flags & ActivityFocus ))
+            {
         // client cannot accept focus, but at least the window should be active (window menu, et. al. )
-        c->setActive( true );
-        focusToNull();
+            c->setActive( true );
+            focusToNull();
+            }
+        flags &= ~ActivityFocus;
+        handled = false; // no point, can't get clicks
         }
+    if( !c->isShown( true )) // shouldn't happen, call activateClient() if needed
+        {
+        kdWarning( 1212 ) << "takeActivity: not shown" << endl;
+        return;
+        }
+    c->takeActivity( flags, handled, Allowed );
+    }
+
+void Workspace::handleTakeActivity( Client* c, Time /*timestamp*/, int flags )
+    {
+    if( pending_take_activity != c ) // pending_take_activity is reset when doing restack or activation
+        return;
+    if(( flags & ActivityRaise ) != 0 )
+        raiseClient( c );
+    if(( flags & ActivityFocus ) != 0 && c->isShown( false ))
+        c->takeFocus( Allowed );
+    pending_take_activity = NULL;
     }
 
 /*!
@@ -351,32 +384,29 @@ void Workspace::clientHidden( Client* c )
     activateNextClient( c );
     }
 
-// deactivates 'c' and activates next client    
-void Workspace::activateNextClient( Client* c )
+// deactivates 'c' and activates next client
+bool Workspace::activateNextClient( Client* c )
     {
     // if 'c' is not the active or the to-become active one, do nothing
     if( !( c == active_client
             || ( should_get_focus.count() > 0 && c == should_get_focus.last())))
-        return;
-    if( popup )
-        popup->close();
-    if( c == active_client )
-        setActiveClient( NULL, Allowed );
-    should_get_focus.remove( c );
+        return false;
+    closeActivePopup();
+    if( c != NULL )
+        {
+        if( c == active_client )
+            setActiveClient( NULL, Allowed );
+        should_get_focus.remove( c );
+        }
     if( focusChangeEnabled())
         {
-        if ( c->wantsTabFocus() && focus_chain.contains( c ) )
-            {
-            focus_chain.remove( c );
-            focus_chain.prepend( c );
-            }
         if ( options->focusPolicyIsReasonable())
             { // search the focus_chain for a client to transfer focus to
-          // if 'c' is transient, transfer focus to the first suitable mainwindow
+              // if 'c' is transient, transfer focus to the first suitable mainwindow
             Client* get_focus = NULL;
-            const ClientList mainwindows = c->mainClients();
-            for( ClientList::ConstIterator it = focus_chain.fromLast();
-                 it != focus_chain.end();
+            const ClientList mainwindows = ( c != NULL ? c->mainClients() : ClientList());
+            for( ClientList::ConstIterator it = focus_chain[currentDesktop()].fromLast();
+                 it != focus_chain[currentDesktop()].end();
                  --it )
                 {
                 if( !(*it)->isShown( false ) || !(*it)->isOnCurrentDesktop())
@@ -396,11 +426,14 @@ void Workspace::activateNextClient( Client* c )
             else
                 focusToNull();
             }
+            else
+                return false;
         }
     else
         // if blocking focus, move focus to the desktop later if needed
         // in order to avoid flickering
         focusToNull();
+    return true;
     }
 
 
@@ -415,10 +448,15 @@ void Workspace::gotFocusIn( const Client* c )
         }
     }
 
+void Workspace::setShouldGetFocus( Client* c )
+    {
+    should_get_focus.append( c );
+    updateStackingOrder(); // e.g. fullscreens have different layer when active/not-active
+    }
 
 // focus_in -> the window got FocusIn event
 // session_active -> the window was active when saving session
-bool Workspace::allowClientActivation( const Client* c, Time time, bool focus_in, bool session_active )
+bool Workspace::allowClientActivation( const Client* c, Time time, bool focus_in )
     {
     // options->focusStealingPreventionLevel :
     // 0 - none    - old KWin behaviour, new windows always get focus
@@ -430,8 +468,8 @@ bool Workspace::allowClientActivation( const Client* c, Time time, bool focus_in
     // 4 - extreme - no window gets focus without user intervention
     if( time == -1U )
         time = c->userTime();
-    if( session_saving
-        && options->focusStealingPreventionLevel <= 2 ) // <= normal
+    int level = c->rules()->checkFSP( options->focusStealingPreventionLevel );
+    if( session_saving && level <= 2 ) // <= normal
         {
         return true;
         }
@@ -444,41 +482,40 @@ bool Workspace::allowClientActivation( const Client* c, Time time, bool focus_in
         // got FocusOut, and therefore got deactivated.
         ac = last_active_client;
         }
-    if( options->focusStealingPreventionLevel == 0 ) // none
-        return true;
-    if( options->focusStealingPreventionLevel == 4 ) // extreme
+    if( time == 0 ) // explicitly asked not to get focus
         return false;
+    if( level == 0 ) // none
+        return true;
+    if( level == 4 ) // extreme
+        return false;
+    if( !c->isOnCurrentDesktop())
+        return false; // allow only with level == 0
+    if( c->ignoreFocusStealing())
+        return true;
     if( ac == NULL || ac->isDesktop())
         {
         kdDebug( 1212 ) << "Activation: No client active, allowing" << endl;
         return true; // no active client -> always allow
         }
-    if( c->ignoreFocusStealing())
-        return true;
-    if( time == 0 ) // explicitly asked not to get focus
-        return false;
     // TODO window urgency  -> return true?
     if( Client::belongToSameApplication( c, ac, true ))
         {
         kdDebug( 1212 ) << "Activation: Belongs to active application" << endl;
         return true;
         }
-    if( options->focusStealingPreventionLevel == 3 ) // high
+    if( level == 3 ) // high
         return false;
     if( time == -1U )  // no time known
-        if( session_active )
-            return !was_user_interaction; // see Client::readUserTimeMapTimestamp()
-        else
         {
-        kdDebug() << "Activation: No timestamp at all" << endl;
-        if( options->focusStealingPreventionLevel == 1 ) // low
+        kdDebug( 1212 ) << "Activation: No timestamp at all" << endl;
+        if( level == 1 ) // low
             return true;
         // no timestamp at all, don't activate - because there's also creation timestamp
         // done on CreateNotify, this case should happen only in case application
         // maps again already used window, i.e. this won't happen after app startup
         return false; 
         }
-    // options->focusStealingPreventionLevel == 2 // normal
+    // level == 2 // normal
     Time user_time = ac->userTime();
     kdDebug( 1212 ) << "Activation, compared:" << c << ":" << time << ":" << user_time
         << ":" << ( timestampCompare( time, user_time ) >= 0 ) << endl;
@@ -489,17 +526,17 @@ bool Workspace::allowClientActivation( const Client* c, Time time, bool focus_in
 // a window to be fully raised upon its own request (XRaiseWindow),
 // if refused, it will be raised only on top of windows belonging
 // to the same application
-bool Workspace::allowFullClientRaising( const Client* c )
+bool Workspace::allowFullClientRaising( const Client* c, Time time )
     {
-    if( session_saving
-        && options->focusStealingPreventionLevel <= 2 ) // <= normal
+    int level = c->rules()->checkFSP( options->focusStealingPreventionLevel );
+    if( session_saving && level <= 2 ) // <= normal
         {
         return true;
         }
     Client* ac = mostRecentlyActivatedClient();
-    if( options->focusStealingPreventionLevel == 0 ) // none
+    if( level == 0 ) // none
         return true;
-    if( options->focusStealingPreventionLevel == 4 ) // extreme
+    if( level == 4 ) // extreme
         return false;
     if( ac == NULL || ac->isDesktop())
         {
@@ -514,17 +551,12 @@ bool Workspace::allowFullClientRaising( const Client* c )
         kdDebug( 1212 ) << "Raising: Belongs to active application" << endl;
         return true;
         }
-    if( options->focusStealingPreventionLevel == 3 ) // high
+    if( level == 3 ) // high
         return false;
-    if( !c->hasUserTimeSupport())
-        {
-        kdDebug() << "Raising: No support" << endl;
-        if( options->focusStealingPreventionLevel == 1 ) // low
-            return true;
-        }
-    // options->focusStealingPreventionLevel == 2 // normal
-    kdDebug() << "Raising: Refusing" << endl;
-    return false;
+    Time user_time = ac->userTime();
+    kdDebug( 1212 ) << "Raising, compared:" << time << ":" << user_time
+        << ":" << ( timestampCompare( time, user_time ) >= 0 ) << endl;
+    return timestampCompare( time, user_time ) >= 0; // time >= user_time
     }
 
 // called from Client after FocusIn that wasn't initiated by KWin and the client
@@ -598,6 +630,7 @@ void Client::updateUserTime( Time time )
         && ( user_time == CurrentTime
             || timestampCompare( time, user_time ) > 0 )) // time > user_time
         user_time = time;
+    group()->updateUserTime( user_time );
     }
 
 Time Client::readUserCreationTime() const
@@ -625,10 +658,44 @@ void Client::demandAttention( bool set )
     {
     if( isActive())
         set = false;
-    info->setState( set ? NET::DemandsAttention : 0, NET::DemandsAttention );
+    if( demands_attention == set )
+        return;
+    demands_attention = set;
+    if( demands_attention )
+        {
+        // Demand attention flag is often set right from manage(), when focus stealing prevention
+        // steps in. At that time the window has no taskbar entry yet, so KNotify cannot place
+        // e.g. the passive popup next to it. So wait up to 1 second for the icon geometry
+        // to be set.
+        // Delayed call to KNotify also solves the problem of having X server grab in manage(),
+        // which may deadlock when KNotify (or KLauncher when launching KNotify) need to access X.
+        Notify::Event e = isOnCurrentDesktop() ? Notify::DemandAttentionCurrent : Notify::DemandAttentionOther;
+        // Setting the demands attention state needs to be done directly in KWin, because
+        // KNotify would try to set it, resulting in a call to KNotify again, etc.
+        if( Notify::makeDemandAttention( e ))
+            info->setState( set ? NET::DemandsAttention : 0, NET::DemandsAttention );
+
+        if( demandAttentionKNotifyTimer == NULL )
+            {
+            demandAttentionKNotifyTimer = new QTimer( this );
+            connect( demandAttentionKNotifyTimer, SIGNAL( timeout()), SLOT( demandAttentionKNotify()));
+            }
+        demandAttentionKNotifyTimer->start( 1000, true );
+        }
+    else
+        info->setState( set ? NET::DemandsAttention : 0, NET::DemandsAttention );
     workspace()->clientAttentionChanged( this, set );
     }
 
+void Client::demandAttentionKNotify()
+    {
+    Notify::Event e = isOnCurrentDesktop() ? Notify::DemandAttentionCurrent : Notify::DemandAttentionOther;
+    Notify::raise( e, i18n( "Window '%1' demands attention." ).arg( KStringHandler::csqueeze(caption())), this );
+    demandAttentionKNotifyTimer->stop();
+    demandAttentionKNotifyTimer->deleteLater();
+    demandAttentionKNotifyTimer = NULL;
+    }
+
 // TODO I probably shouldn't be lazy here and do it without the macro, so that people can read it
 KWIN_COMPARE_PREDICATE( SameApplicationActiveHackPredicate, const Client*,
     // ignore already existing splashes, toolbars, utilities, menus and topmenus,
@@ -636,18 +703,27 @@ KWIN_COMPARE_PREDICATE( SameApplicationActiveHackPredicate, const Client*,
     !cl->isSplash() && !cl->isToolbar() && !cl->isTopMenu() && !cl->isUtility() && !cl->isMenu()
     && Client::belongToSameApplication( cl, value, true ) && cl != value);
 
-Time Client::readUserTimeMapTimestamp( const KStartupInfoData* asn_data,
-    const SessionInfo* session ) const
+Time Client::readUserTimeMapTimestamp( const KStartupInfoId* asn_id, const KStartupInfoData* asn_data,
+    bool session ) const
     {
     Time time = info->userTime();
     kdDebug( 1212 ) << "User timestamp, initial:" << time << endl;
     // newer ASN timestamp always replaces user timestamp, unless user timestamp is 0
     // helps e.g. with konqy reusing
-    if( asn_data != NULL && time != 0
-        && ( time == -1U
-            || ( asn_data->timestamp() != -1U
-                && timestampCompare( asn_data->timestamp(), time ) > 0 )))
-        time = asn_data->timestamp();
+    if( asn_data != NULL && time != 0 )
+        {
+        // prefer timestamp from ASN id (timestamp from data is obsolete way)
+        if( asn_id->timestamp() != 0
+            && ( time == -1U || timestampCompare( asn_id->timestamp(), time ) > 0 ))
+            {
+            time = asn_id->timestamp();
+            }
+        else if( asn_data->timestamp() != -1U
+            && ( time == -1U || timestampCompare( asn_data->timestamp(), time ) > 0 ))
+            {
+            time = asn_data->timestamp();
+            }
+        }
     kdDebug( 1212 ) << "User timestamp, ASN:" << time << endl;
     if( time == -1U )
         { // The window doesn't have any timestamp.
@@ -656,7 +732,7 @@ Time Client::readUserTimeMapTimestamp( const KStartupInfoData* asn_data,
       // use the _KDE_NET_WM_USER_CREATION_TIME trick.
       // Otherwise, refuse activation of a window
       // from already running application if this application
-      // is not the active one.
+      // is not the active one (unless focus stealing prevention is turned off).
         Client* act = workspace()->mostRecentlyActivatedClient();
         if( act != NULL && !belongToSameApplication( act, this, true ))
             {
@@ -677,7 +753,8 @@ Time Client::readUserTimeMapTimestamp( const KStartupInfoData* asn_data,
                 if( workspace()->findClient( SameApplicationActiveHackPredicate( this )))
                     first_window = false;
                 }
-            if( !first_window )
+            // don't refuse if focus stealing prevention is turned off
+            if( !first_window && rules()->checkFSP( options->focusStealingPreventionLevel ) > 0 )
                 {
                 kdDebug( 1212 ) << "User timestamp, already exists:" << 0 << endl;
                 return 0; // refuse activation
@@ -691,8 +768,8 @@ Time Client::readUserTimeMapTimestamp( const KStartupInfoData* asn_data,
         // it's better not to activate the new one.
         // Unless it was the active window at the time
         // of session saving and there was no user interaction yet,
-        // this check will be done in Workspace::allowClientActiovationTimestamp().
-        if( session && !session->fake )
+        // this check will be done in manage().
+        if( session )
             return -1U;
         if( ignoreFocusStealing() && act != NULL )
             time = act->userTime();
@@ -706,6 +783,8 @@ Time Client::readUserTimeMapTimestamp( const KStartupInfoData* asn_data,
 Time Client::userTime() const
     {
     Time time = user_time;
+    if( time == 0 ) // doesn't want focus after showing
+        return 0;
     assert( group() != NULL );
     if( time == -1U
          || ( group()->userTime() != -1U
@@ -725,13 +804,21 @@ Time Client::userTime() const
   its own.
 
  */
-void Client::setActive( bool act)
+void Client::setActive( bool act, bool updateOpacity_)
     {
     if ( active == act )
         return;
     active = act;
     workspace()->setActiveClient( act ? this : NULL, Allowed );
-
+    
+    if (updateOpacity_) updateOpacity();
+    if (isModal() && transientFor())
+    {
+        if (!act) transientFor()->updateOpacity();
+        else if (!transientFor()->custom_opacity) transientFor()->setOpacity(options->translucentActiveWindows, options->activeWindowOpacity);
+    }
+    updateShadowSize();
+    
     if ( active )
         Notify::raise( Notify::Activate );
 
@@ -740,7 +827,7 @@ void Client::setActive( bool act)
 
     if( !active && shade_mode == ShadeActivated )
         setShade( ShadeNormal );
-
+        
     StackingUpdatesBlocker blocker( workspace());
     workspace()->updateClientLayer( this ); // active windows may get different layer
     // TODO optimize? mainClients() may be a bit expensive
@@ -758,15 +845,25 @@ void Client::setActive( bool act)
 
 void Client::startupIdChanged()
     {
+    KStartupInfoId asn_id;
     KStartupInfoData asn_data;
-    bool asn_valid = workspace()->checkStartupNotification( window(), asn_data );
+    bool asn_valid = workspace()->checkStartupNotification( window(), asn_id, asn_data );
     if( !asn_valid )
         return;
+    // If the ASN contains desktop, move it to the desktop, otherwise move it to the current
+    // desktop (since the new ASN should make the window act like if it's a new application
+    // launched). However don't affect the window's desktop if it's set to be on all desktops.
+    int desktop = workspace()->currentDesktop();
     if( asn_data.desktop() != 0 )
-        workspace()->sendClientToDesktop( this, asn_data.desktop(), true );
-    if( asn_data.timestamp() != -1U )
+        desktop = asn_data.desktop();
+    if( !isOnAllDesktops())
+        workspace()->sendClientToDesktop( this, desktop, true );
+    Time timestamp = asn_id.timestamp();
+    if( timestamp == 0 && asn_data.timestamp() != -1U )
+        timestamp = asn_data.timestamp();
+    if( timestamp != 0 )
         {
-        bool activate = workspace()->allowClientActivation( this, asn_data.timestamp());
+        bool activate = workspace()->allowClientActivation( this, timestamp );
         if( asn_data.desktop() != 0 && !isOnCurrentDesktop())
             activate = false; // it was started on different desktop than current one
         if( activate )
@@ -782,19 +879,32 @@ void Client::updateUrgency()
         demandAttention();
     }
 
+void Client::shortcutActivated()
+    {
+    workspace()->activateClient( this, true ); // force
+    }
+
 //****************************************
 // Group
 //****************************************
     
 void Group::startupIdChanged()
     {
+    KStartupInfoId asn_id;
     KStartupInfoData asn_data;
-    bool asn_valid = workspace()->checkStartupNotification( leader_wid, asn_data );
+    bool asn_valid = workspace()->checkStartupNotification( leader_wid, asn_id, asn_data );
     if( !asn_valid )
         return;
-    if( asn_data.timestamp() != -1U && user_time != -1U
-        &&timestampCompare( asn_data.timestamp(), user_time ) > 0 )
+    if( asn_id.timestamp() != 0 && user_time != -1U
+        && timestampCompare( asn_id.timestamp(), user_time ) > 0 )
+        {
+        user_time = asn_id.timestamp();
+        }
+    else if( asn_data.timestamp() != -1U && user_time != -1U
+        && timestampCompare( asn_data.timestamp(), user_time ) > 0 )
+        {
         user_time = asn_data.timestamp();
+        }
     }
 
 void Group::updateUserTime( Time time )
