@@ -92,7 +92,8 @@ Client::Client( Workspace *ws )
         border_left( 0 ),
         border_right( 0 ),
         border_top( 0 ),
-        border_bottom( 0 )
+        border_bottom( 0 ),
+        demandAttentionKNotifyTimer( NULL )
 // SELI do all as initialization
     {
     autoRaiseTimer = 0;
@@ -110,6 +111,7 @@ Client::Client( Workspace *ws )
 
     shade_mode = ShadeNone;
     active = FALSE;
+    deleting = false;
     keep_above = FALSE;
     keep_below = FALSE;
     is_shape = FALSE;
@@ -128,6 +130,7 @@ Client::Client( Workspace *ws )
     not_obscured = false;
     urgency = false;
     ignore_focus_stealing = false;
+    demands_attention = false;
     check_active_modal = false;
 
     Pdeletewindow = 0;
@@ -139,7 +142,8 @@ Client::Client( Workspace *ws )
     skip_pager = FALSE;
 
     max_mode = MaximizeRestore;
-
+    maxmode_restore = MaximizeRestore;
+    
     cmap = None;
     
     frame_geometry = QRect( 0, 0, 100, 100 ); // so that decorations don't start with size being (0,0)
@@ -177,6 +181,9 @@ void Client::deleteClient( Client* c, allowed_t )
  */
 void Client::releaseWindow( bool on_shutdown )
     {
+    assert( !deleting );
+    deleting = true;
+    workspace()->discardUsedWindowRules( this, true ); // remove ForceTemporarily rules
     StackingUpdatesBlocker blocker( workspace());
     if (!custom_opacity) setOpacity(FALSE);
     if (moveResizeMode)
@@ -200,8 +207,9 @@ void Client::releaseWindow( bool on_shutdown )
         desk = 0;
         info->setState( 0, info->state()); // reset all state flags
         }
-    XDeleteProperty( qt_xdisplay(),  client, atoms->kde_net_wm_user_creation_time);
-    // TODO remove KDEFrameStrut property
+    XDeleteProperty( qt_xdisplay(), client, atoms->kde_net_wm_user_creation_time);
+    XDeleteProperty( qt_xdisplay(), client, atoms->net_frame_extents );
+    XDeleteProperty( qt_xdisplay(), client, atoms->kde_net_wm_frame_strut );
     XReparentWindow( qt_xdisplay(), client, workspace()->rootWin(), x(), y());
     XRemoveFromSaveSet( qt_xdisplay(), client );
     XSelectInput( qt_xdisplay(), client, NoEventMask );
@@ -229,6 +237,9 @@ void Client::releaseWindow( bool on_shutdown )
 // (e.g. the application closed it)
 void Client::destroyClient()
     {
+    assert( !deleting );
+    deleting = true;
+    workspace()->discardUsedWindowRules( this, true ); // remove ForceTemporarily rules
     StackingUpdatesBlocker blocker( workspace());
     if (moveResizeMode)
        leaveMoveResize();
@@ -267,7 +278,9 @@ void Client::updateDecoration( bool check_workspace_pos, bool force )
         XReparentWindow( qt_xdisplay(), decoration->widget()->winId(), frameId(), 0, 0 );
         decoration->widget()->lower();
         decoration->borders( border_left, border_right, border_top, border_bottom );
-        setXTitleHeightProperty(border_top);
+        options->onlyDecoTranslucent ?
+            setDecoHashProperty(border_top, border_right, border_bottom, border_left):
+            unsetDecoHashProperty();
         int save_workarea_diff_x = workarea_diff_x;
         int save_workarea_diff_y = workarea_diff_y;
         move( calculateGravitation( false ));
@@ -283,7 +296,7 @@ void Client::updateDecoration( bool check_workspace_pos, bool force )
     postponeGeometryUpdates( false );
     if( do_show )
         decoration->widget()->show();
-    updateFrameStrut();
+    updateFrameExtents();
     }
 
 void Client::destroyDecoration()
@@ -317,10 +330,15 @@ void Client::checkBorderSizes()
     move( calculateGravitation( true ));
     border_left = new_left;
     border_right = new_right;
-    if (border_top != new_top)
-        setXTitleHeightProperty(new_top);
     border_top = new_top;
     border_bottom = new_bottom;
+    if (border_left != new_left ||
+        border_right != new_right ||
+        border_top != new_top ||
+        border_bottom != new_bottom)
+    options->onlyDecoTranslucent ?
+       setDecoHashProperty(new_top, new_right, new_bottom, new_left):
+       unsetDecoHashProperty();
     move( calculateGravitation( false ));
     plainResize( sizeForClientSize( clientSize()), ForceGeometrySet );
     checkWorkspacePosition();
@@ -338,7 +356,6 @@ void Client::detectNoBorder()
         case NET::Desktop :
         case NET::Dock :
         case NET::TopMenu :
-        case NET::Override :
         case NET::Splash :
             noborder = true;
           break;
@@ -349,6 +366,34 @@ void Client::detectNoBorder()
         case NET::Dialog :
         case NET::Utility :
             noborder = false;
+          break;
+        default:
+            assert( false );
+        }
+    // NET::Override is some strange beast without clear definition, usually
+    // just meaning "noborder", so let's treat it only as such flag, and ignore it as
+    // a window type otherwise (SUPPORTED_WINDOW_TYPES_MASK doesn't include it)
+    if( info->windowType( SUPPORTED_WINDOW_TYPES_MASK | NET::OverrideMask ) == NET::Override )
+        noborder = true;
+    }
+
+void Client::detectShapable()
+    {
+    if( Shape::hasShape( window()))
+        return;
+    switch( windowType())
+        {
+        case NET::Desktop :
+        case NET::Dock :
+        case NET::TopMenu :
+        case NET::Splash :
+          break;
+        case NET::Unknown :
+        case NET::Normal :
+        case NET::Toolbar :
+        case NET::Menu :
+        case NET::Dialog :
+        case NET::Utility :
             setShapable(FALSE);
           break;
         default:
@@ -356,15 +401,14 @@ void Client::detectNoBorder()
         }
     }
 
-void Client::updateFrameStrut()
+void Client::updateFrameExtents()
     {
-// TODO KDEFrameStrut je ale pitome jmeno
     NETStrut strut;
     strut.left = border_left;
     strut.right = border_right;
     strut.top = border_top;
     strut.bottom = border_bottom;
-    info->setKDEFrameStrut( strut );
+    info->setFrameExtents( strut );
     }
 
 // Resizes the decoration, and makes sure the decoration widget gets resize event
@@ -480,31 +524,15 @@ void Client::hideClient( bool hide )
     if( hidden == hide )
         return;
     hidden = hide;
-    info->setState( hidden ? NET::Hidden : 0, NET::Hidden );
-    if( hidden )
-        {
-        setMappingState( IconicState );
-        rawHide();
-        setSkipTaskbar( true, false ); // also hide from taskbar
-        }
-    else // !hidden
-        {
-        setSkipTaskbar( original_skip_taskbar, false );
-        if( isOnCurrentDesktop())
-            {
-            if( isShown( false ))
-                setMappingState( NormalState );
-            rawShow(); // is either visible or shaded
-            }
-        }
+    updateVisibility();
     }
-
+    
 /*
   Returns whether the window is minimizable or not
  */
 bool Client::isMinimizable() const
     {
-    if( isSpecialWindow() && !isOverride())
+    if( isSpecialWindow())
         return false;
     if( isTransient())
         { // #66868 - let other xmms windows be minimized when the mainwindow is minimized
@@ -538,20 +566,19 @@ void Client::minimize( bool avoid_animation )
     if ( !isMinimizable() || isMinimized())
         return;
 
-    minimized = true;
-
     Notify::raise( Notify::Minimize );
 
     // SELI mainClients().isEmpty() ??? - and in unminimize() too
-    if ( mainClients().isEmpty() && isOnCurrentDesktop() && !avoid_animation )
+    if ( mainClients().isEmpty() && isOnCurrentDesktop() && isShown( true ) && !avoid_animation )
         animateMinimizeOrUnminimize( true ); // was visible or shaded
 
-    setMappingState( IconicState );
-    info->setState( NET::Hidden, NET::Hidden );
-    rawHide();
+    minimized = true;
+
+    updateVisibility();
     updateAllowedActions();
     workspace()->updateMinimizedOfTransients( this );
     updateWindowRules();
+    workspace()->updateFocusChains( this, false ); // make it last in the focus chain
     }
 
 void Client::unminimize( bool avoid_animation )
@@ -560,16 +587,13 @@ void Client::unminimize( bool avoid_animation )
         return;
 
     Notify::raise( Notify::UnMinimize );
-    minimized = false;    
-    info->setState( 0, NET::Hidden );
-    if( isOnCurrentDesktop())
+    minimized = false;
+    if( isOnCurrentDesktop() && isShown( true ))
         {
         if( mainClients().isEmpty() && !avoid_animation )
             animateMinimizeOrUnminimize( FALSE );
-        if( isShown( false ))
-            setMappingState( NormalState );
-        rawShow(); // is either visible or shaded
         }
+    updateVisibility();
     updateAllowedActions();
     workspace()->updateMinimizedOfTransients( this );
     updateWindowRules();
@@ -812,7 +836,7 @@ void Client::setShade( ShadeMode mode )
     checkMaximizeGeometry();
     info->setState( isShade() ? NET::Shaded : 0, NET::Shaded );
     info->setState( isShown( false ) ? 0 : NET::Hidden, NET::Hidden );
-    setMappingState( isShown( false ) && isOnCurrentDesktop() ? NormalState : IconicState );
+    updateVisibility();
     updateAllowedActions();
     workspace()->updateMinimizedOfTransients( this );
     decoration->shadeChange();
@@ -822,6 +846,11 @@ void Client::setShade( ShadeMode mode )
 void Client::shadeHover()
     {
     setShade( ShadeHover );
+    cancelShadeHover();
+    }
+
+void Client::cancelShadeHover()
+    {
     delete shadeHoverTimer;
     shadeHoverTimer = 0;
     }
@@ -832,22 +861,47 @@ void Client::toggleShade()
     setShade( shade_mode == ShadeNone ? ShadeNormal : ShadeNone );
     }
 
-void Client::virtualDesktopChange()
+void Client::updateVisibility()
     {
-    if( hidden || minimized )
-        return; // no visibility change
-    // from here it can be only shaded or normally shown
-    if( isOnCurrentDesktop())
+    if( deleting )
+        return;
+    bool show = true;
+    if( hidden )
         {
-        if( !isShade())
-            setMappingState( NormalState );
-        rawShow();
+        setMappingState( IconicState );
+        info->setState( NET::Hidden, NET::Hidden );
+        setSkipTaskbar( true, false ); // also hide from taskbar
+        rawHide();
+        show = false;
         }
     else
         {
-        if( !isShade())
-            setMappingState( IconicState );
+        setSkipTaskbar( original_skip_taskbar, false );
+        }
+    if( minimized )
+        {
+        setMappingState( IconicState );
+        info->setState( NET::Hidden, NET::Hidden );
         rawHide();
+        show = false;
+        }
+    if( show )
+        info->setState( 0, NET::Hidden );
+    if( !isOnCurrentDesktop())
+        {
+        setMappingState( IconicState );
+        rawHide();
+        show = false;
+        }
+    if( show )
+        {
+        if( workspace()->showingDesktop())
+            workspace()->resetShowingDesktop( true );
+        if( isShade())
+            setMappingState( IconicState );
+        else
+            setMappingState( NormalState );
+        rawShow();
         }
     }
 
@@ -858,6 +912,7 @@ void Client::virtualDesktopChange()
 void Client::setMappingState(int s)
     {
     assert( client != None );
+    assert( !deleting || s == WithdrawnState );
     if( mapping_state == s )
         return;
     bool was_unmanaged = ( mapping_state == WithdrawnState );
@@ -944,7 +999,7 @@ void Client::sendClientMessage(Window w, Atom a, Atom protocol, long data1, long
  */
 bool Client::isCloseable() const
     {
-    return rules()->checkCloseable( motif_may_close && ( !isSpecialWindow() || isOverride())); // TODO is NET::Override special?
+    return rules()->checkCloseable( motif_may_close && !isSpecialWindow());
     }
 
 /*!
@@ -955,11 +1010,8 @@ void Client::closeWindow()
     {
     if( !isCloseable())
         return;
-    // Update user time, needed for whole group, because the window may create a confirming dialog,
-    // and this window's user time wouldn't apply to it
-    // This is needed even for apps without support for user timestamp (e.g. nedit), so updating
-    // user timestamp in apps on WM_DELETE_WINDOW is not an option (and I'm not sure if it would be right)
-    group()->updateUserTime(); 
+    // Update user time, because the window may create a confirming dialog.
+    updateUserTime(); 
     if ( Pdeletewindow )
         {
         Notify::raise( Notify::Close );
@@ -1085,6 +1137,7 @@ void Client::processKillerExited()
 
 void Client::setSkipTaskbar( bool b, bool from_outside )
     {
+    int was_wants_tab_focus = wantsTabFocus();
     if( from_outside )
         {
         b = rules()->checkSkipTaskbar( b );
@@ -1095,6 +1148,8 @@ void Client::setSkipTaskbar( bool b, bool from_outside )
     skip_taskbar = b;
     info->setState( b?NET::SkipTaskbar:0, NET::SkipTaskbar );
     updateWindowRules();
+    if( was_wants_tab_focus != wantsTabFocus())
+        workspace()->updateFocusChains( this, isActive());
     }
 
 void Client::setSkipPager( bool b )
@@ -1136,7 +1191,8 @@ void Client::setDesktop( int desktop )
         }
     if( decoration != NULL )
         decoration->desktopChange();
-    virtualDesktopChange(); // hide/show if needed
+    workspace()->updateFocusChains( this, true );
+    updateVisibility();
     updateWindowRules();
     }
 
@@ -1173,7 +1229,7 @@ void Client::takeActivity( int flags, bool handled, allowed_t )
     static Client* previous_client;
     if( previous_activity_timestamp == qt_x_time && previous_client != this )
         {
-        kdWarning( 1212 ) << "Repeated use of the same X timestamp for activity" << endl;
+        kdDebug( 1212 ) << "Repeated use of the same X timestamp for activity" << endl;
         kdDebug( 1212 ) << kdBacktrace() << endl;
         }
     previous_activity_timestamp = qt_x_time;
@@ -1190,7 +1246,7 @@ void Client::takeFocus( allowed_t )
     static Client* previous_client;
     if( previous_focus_timestamp == qt_x_time && previous_client != this )
         {
-        kdWarning( 1212 ) << "Repeated use of the same X timestamp for focus" << endl;
+        kdDebug( 1212 ) << "Repeated use of the same X timestamp for focus" << endl;
         kdDebug( 1212 ) << kdBacktrace() << endl;
         }
     previous_focus_timestamp = qt_x_time;
@@ -1358,6 +1414,8 @@ void Client::getMotifHints()
         motif_may_resize = mresize; // this should be set using minsize==maxsize, but oh well
         motif_may_move = mmove;
         }
+    else
+        motif_may_resize = motif_may_move = true;
     // mminimize; - ignore, bogus - e.g. shading or sending to another desktop is "minimizing" too
     // mmaximize; - ignore, bogus - maximizing is basically just resizing
     motif_may_close = mclose; // motif apps like to crash when they set this hint and WM closes them anyway
@@ -1556,7 +1614,7 @@ Window Client::wmClientLeader() const
 
 bool Client::wantsTabFocus() const
     {
-    return ( isNormalWindow() || isDialog() || isOverride()) && wantsInput() && !skip_taskbar;
+    return ( isNormalWindow() || isDialog()) && wantsInput() && !skip_taskbar;
     }
 
 
@@ -1591,11 +1649,6 @@ bool Client::isToolbar() const
     return windowType() == NET::Toolbar;
     }
 
-bool Client::isOverride() const
-    {
-    return windowType() == NET::Override;
-    }
-
 bool Client::isSplash() const
     {
     return windowType() == NET::Splash;
@@ -1619,7 +1672,6 @@ bool Client::isNormalWindow() const
 bool Client::isSpecialWindow() const
     {
     return isDesktop() || isDock() || isSplash() || isTopMenu()
-        || ( isOverride() && !isFullScreen())// SELI is NET::Override special or not?
         || isToolbar(); // TODO
     }
 
@@ -1962,11 +2014,12 @@ bool Client::getWindowOpacity() //query translucency settings from X, returns tr
     int format, result;
     unsigned long n, left;
     result = XGetWindowProperty(qt_xdisplay(), window(), atoms->net_wm_window_opacity, 0L, 1L, False, XA_CARDINAL, &actual, &format, &n, &left, /*(unsigned char **)*/ &data);
-    if (result == Success && data != None)
+    if (result == Success && data != None && format == 32 )
         {
-        memcpy (&opacity_, data, sizeof (unsigned int));
+        opacity_ = *reinterpret_cast< long* >( data );
         custom_opacity = true;
 //         setOpacity(opacity_ < 0xFFFFFFFF, opacity_);
+        XFree ((char*)data);
         return TRUE;
         }
     return FALSE;
@@ -2001,10 +2054,18 @@ bool Client::touches(const Client* c)
     return FALSE;
     }
     
-void Client::setXTitleHeightProperty(int titleHeight)
+void Client::setDecoHashProperty(uint topHeight, uint rightWidth, uint bottomHeight, uint leftWidth)
 {
-    long data = titleHeight;
-    XChangeProperty(qt_xdisplay(), frameId(), atoms->net_wm_window_titleheight, XA_CARDINAL, 32, PropModeReplace, (unsigned char *) &data, 1L);
+   long data = (topHeight < 255 ? topHeight : 255) << 24 |
+               (rightWidth < 255 ? rightWidth : 255) << 16 |
+               (bottomHeight < 255 ? bottomHeight : 255) << 8 |
+               (leftWidth < 255 ? leftWidth : 255);
+    XChangeProperty(qt_xdisplay(), frameId(), atoms->net_wm_window_decohash, XA_CARDINAL, 32, PropModeReplace, (unsigned char *) &data, 1L);
+}
+
+void Client::unsetDecoHashProperty()
+{
+   XDeleteProperty( qt_xdisplay(), frameId(), atoms->net_wm_window_decohash);
 }
     
 #ifndef NDEBUG
