@@ -357,6 +357,17 @@ void ShortcutDialog::accept()
         }
     KShortcutDialog::accept();
     }
+
+// Workaround for Qt bug causing #119142 - wheel event causes only calling
+// of hide() but not close(), so dialog closing is not performed.
+// Possible recursive calling close->hide->close should be fine, as close()
+// has checks against that.
+void ShortcutDialog::hide()
+    {
+    close();
+    return KShortcutDialog::hide();
+    }
+
 #endif
 
 
