@@ -1,5 +1,3 @@
-#include <stdlib.h>
-
 #include <qlayout.h>
 #include <qlabel.h>
 #include <qcombobox.h>
@@ -26,20 +24,48 @@
 #include <kiconloader.h>
 
 #include "extension.h"
+#include "kxkbconfig.h"
 #include "rules.h"
-#include "kcmlayout.h"
-#include "kcmlayout.moc"
 #include "pixmap.h"
 #include "kcmmisc.h"
 #include "kcmlayoutwidget.h"
 
-#include <X11/Xlib.h>
+#include "kcmlayout.h"
+#include "kcmlayout.moc"
 
 
-static const char* switchModes[] = {
-  "Global", "WinClass", "Window"
+enum {
+ LAYOUT_COLUMN_FLAG = 0,
+ LAYOUT_COLUMN_NAME = 1,
+ LAYOUT_COLUMN_MAP = 2,
+ LAYOUT_COLUMN_VARIANT = 3,
+ LAYOUT_COLUMN_INCLUDE = 4,
+ SRC_LAYOUT_COLUMN_COUNT = 3,
+ DST_LAYOUT_COLUMN_COUNT = 5
 };
 
+static const QString DEFAULT_VARIANT_NAME("<default>");
+
+
+class OptionListItem : public QCheckListItem
+{
+	public:
+
+		OptionListItem(  OptionListItem *parent, const QString &text, Type tt,
+						 const QString &optionName );
+		OptionListItem(  QListView *parent, const QString &text, Type tt,
+						 const QString &optionName );
+		~OptionListItem() {}
+
+		QString optionName() const { return m_OptionName; }
+
+		OptionListItem *findChildItem(  const QString& text );
+
+	protected:
+		QString m_OptionName;
+};
+
+
 static QString lookupLocalized(const QDict<char> &dict, const QString& text)
 {
   QDictIterator<char> it(dict);
@@ -53,32 +79,20 @@ static QString lookupLocalized(const QDict<char> &dict, const QString& text)
   return QString::null;
 }
 
-OptionListItem::OptionListItem( OptionListItem *parent, const QString &text,
-    Type tt, const QString &optionName )
-    : QCheckListItem( parent, text, tt ), m_OptionName( optionName )
+static QListViewItem* copyLVI(const QListViewItem* src, QListView* parent)
 {
-}
-
-OptionListItem::OptionListItem( QListView *parent, const QString &text,
-    Type tt, const QString &optionName )
-    : QCheckListItem( parent, text, tt ), m_OptionName( optionName )
-{
-}
-
-OptionListItem * OptionListItem::findChildItem( const QString& optionName )
-{
-  OptionListItem *child = static_cast<OptionListItem *>( firstChild() );
-
-  while ( child )
+    QListViewItem* ret = new QListViewItem(parent);
+	for(int i = 0; i < SRC_LAYOUT_COLUMN_COUNT; i++)
     {
-      if ( child->optionName() == optionName )
-        break;
-      child = static_cast<OptionListItem *>( child->nextSibling() );
+        ret->setText(i, src->text(i));
+        if ( src->pixmap(i) )
+            ret->setPixmap(i, *src->pixmap(i));
     }
 
-  return child;
+    return ret;
 }
 
+
 LayoutConfig::LayoutConfig(QWidget *parent, const char *name)
   : KCModule(parent, name), 
     m_rules(NULL)
@@ -118,8 +132,17 @@ LayoutConfig::LayoutConfig(QWidget *parent, const char *name)
   connect( widget->chkEnableSticky, SIGNAL(toggled(bool)), this, SLOT(changed()));
   connect( widget->spinStickyDepth, SIGNAL(valueChanged(int)), this, SLOT(changed()));
 
-  widget->listLayoutsSrc->setColumnWidth(0, 28);
-  widget->listLayoutsDst->setColumnWidth(0, 28);
+  widget->listLayoutsSrc->setColumnText(LAYOUT_COLUMN_FLAG, "");
+  widget->listLayoutsDst->setColumnText(LAYOUT_COLUMN_FLAG, "");
+  widget->listLayoutsDst->setColumnText(LAYOUT_COLUMN_INCLUDE, "");
+
+  widget->listLayoutsSrc->setColumnWidth(LAYOUT_COLUMN_FLAG, 28);
+  widget->listLayoutsDst->setColumnWidth(LAYOUT_COLUMN_FLAG, 28);
+
+  widget->listLayoutsDst->header()->setResizeEnabled(FALSE, LAYOUT_COLUMN_INCLUDE);
+  widget->listLayoutsDst->setColumnWidthMode(LAYOUT_COLUMN_INCLUDE, QListView::Manual);
+  widget->listLayoutsDst->setColumnWidth(LAYOUT_COLUMN_INCLUDE, 0);
+  
   widget->listLayoutsDst->setSorting(-1);
 #if 0
   widget->listLayoutsDst->setResizeMode(QListView::LastColumn);
@@ -127,7 +150,7 @@ LayoutConfig::LayoutConfig(QWidget *parent, const char *name)
 #endif
 
   //Read rules - we _must_ read _before_ creating xkb-options comboboxes
-  ruleChanged();
+  loadRules();
 
   makeOptionsTab();
 
@@ -137,51 +160,215 @@ LayoutConfig::LayoutConfig(QWidget *parent, const char *name)
 
 LayoutConfig::~LayoutConfig()
 {
-  delete m_rules;
+	delete m_rules;
 }
 
+
+void LayoutConfig::load()
+{
+	m_kxkbConfig.load(KxkbConfig::LOAD_ALL);
+
+	initUI();
+}
+	
+void LayoutConfig::initUI() {
+	const char* modelName = m_rules->models()[m_kxkbConfig.m_model];
+	if( modelName == NULL )
+		modelName = DEFAULT_MODEL;
+	
+	widget->comboModel->setCurrentText(i18n(modelName));
+
+	QValueList<LayoutUnit> otherLayouts = m_kxkbConfig.m_layouts;
+// to optimize we should have gone from it.end to it.begin
+	QValueList<LayoutUnit>::ConstIterator it;
+	for (it = otherLayouts.begin(); it != otherLayouts.end(); ++it ) {
+		QListViewItemIterator src_it( widget->listLayoutsSrc );
+		LayoutUnit layoutUnit = *it;
+		
+		for ( ; src_it.current(); ++src_it ) {
+			QListViewItem* srcItem = src_it.current();
+			
+			if ( layoutUnit.layout == src_it.current()->text(LAYOUT_COLUMN_MAP) ) {	// check if current config knows about this layout
+				QListViewItem* newItem = copyLVI(srcItem, widget->listLayoutsDst);
+				
+				newItem->setText(LAYOUT_COLUMN_VARIANT, layoutUnit.variant);
+				newItem->setText(LAYOUT_COLUMN_INCLUDE, layoutUnit.includeGroup);
+				widget->listLayoutsDst->insertItem(newItem);
+				newItem->moveItem(widget->listLayoutsDst->lastItem());
+
+// 				m_includes[ layoutUnit ] = layoutUnit.includeGroup;
+				break;
+			}
+		}
+	}
+
+	// display KXKB switching options
+	widget->chkShowSingle->setChecked(m_kxkbConfig.m_showSingle);
+	widget->chkShowFlag->setChecked(m_kxkbConfig.m_showFlag);
+
+	widget->chkEnableOptions->setChecked( m_kxkbConfig.m_enableXkbOptions );
+	widget->checkResetOld->setChecked(m_kxkbConfig.m_resetOldOptions);
+
+	switch( m_kxkbConfig.m_switchingPolicy ) {
+		default:
+		case SWITCH_POLICY_GLOBAL:
+			widget->grpSwitching->setButton(0);
+			break;
+		case SWITCH_POLICY_WIN_CLASS:
+			widget->grpSwitching->setButton(1);
+			break;
+		case SWITCH_POLICY_WINDOW:
+			widget->grpSwitching->setButton(2);
+			break;
+	}
+
+	widget->chkEnableSticky->setChecked(m_kxkbConfig.m_stickySwitching);
+	widget->spinStickyDepth->setEnabled(m_kxkbConfig.m_stickySwitching);
+	widget->spinStickyDepth->setValue( m_kxkbConfig.m_stickySwitchingDepth);
+
+	updateStickyLimit();
+
+	widget->chkEnable->setChecked( m_kxkbConfig.m_useKxkb );
+	widget->grpLayouts->setEnabled( m_kxkbConfig.m_useKxkb );
+	widget->optionsFrame->setEnabled( m_kxkbConfig.m_useKxkb );
+
+	// display xkb options
+	QStringList options = QStringList::split(',', m_kxkbConfig.m_options);
+	for (QStringList::ConstIterator it = options.begin(); it != options.end(); ++it)
+	{
+		QString option = *it;
+		QString optionKey = option.mid(0, option.find(':'));
+		QString optionName = m_rules->options()[option];
+		OptionListItem *item = m_optionGroups[i18n(optionKey.latin1())];
+		
+		if (item != NULL) {
+			OptionListItem *child = item->findChildItem( option );
+
+			if ( child )
+				child->setState( QCheckListItem::On );
+			else
+				kdDebug() << "load: Unknown option: " << option << endl;
+		}
+		else {
+			kdDebug() << "load: Unknown option group: " << optionKey << " of " << option << endl;
+		}
+	}
+
+	updateOptionsCommand();
+	emit KCModule::changed( false );
+}
+
+
+void LayoutConfig::save()
+{
+	QString model = lookupLocalized(m_rules->models(), widget->comboModel->currentText());
+	m_kxkbConfig.m_model = model;
+
+	m_kxkbConfig.m_enableXkbOptions = widget->chkEnableOptions->isChecked();
+	m_kxkbConfig.m_resetOldOptions = widget->checkResetOld->isChecked();
+	m_kxkbConfig.m_options = createOptionString();
+
+	QListViewItem *item = widget->listLayoutsDst->firstChild();
+	QValueList<LayoutUnit> layouts;
+	while (item) {
+		QString layout = item->text(LAYOUT_COLUMN_MAP);
+		QString variant = item->text(LAYOUT_COLUMN_VARIANT);
+		QString includes = item->text(LAYOUT_COLUMN_INCLUDE);
+		
+		LayoutUnit layoutUnit(layout, variant);
+		layoutUnit.includeGroup = includes;
+		layouts.append( layoutUnit );
+		
+		item = item->nextSibling();
+		kdDebug() << "Tosave: layout " << layoutUnit.toPair() << ", inc: " << layoutUnit.includeGroup << endl;
+	}
+	m_kxkbConfig.m_layouts = layouts;
+
+	if( m_kxkbConfig.m_layouts.count() == 0 ) {
+		m_kxkbConfig.m_layouts.append(LayoutUnit(DEFAULT_LAYOUT_UNIT));
+ 		widget->chkEnable->setChecked(false);
+ 	}
+
+	m_kxkbConfig.m_useKxkb = widget->chkEnable->isChecked();
+	m_kxkbConfig.m_showSingle = widget->chkShowSingle->isChecked();
+	m_kxkbConfig.m_showFlag = widget->chkShowFlag->isChecked();
+
+	int modeId = widget->grpSwitching->id(widget->grpSwitching->selected());
+	switch( modeId ) {
+		default:
+		case 0:
+			m_kxkbConfig.m_switchingPolicy = SWITCH_POLICY_GLOBAL;
+			break;
+		case 1:
+			m_kxkbConfig.m_switchingPolicy = SWITCH_POLICY_WIN_CLASS;
+			break;
+		case 2:
+			m_kxkbConfig.m_switchingPolicy = SWITCH_POLICY_WINDOW;
+			break;
+	}
+
+	m_kxkbConfig.m_stickySwitching = widget->chkEnableSticky->isChecked();
+	m_kxkbConfig.m_stickySwitchingDepth = widget->spinStickyDepth->value();
+
+	m_kxkbConfig.save();
+	
+	kapp->kdeinitExec("kxkb");
+	emit KCModule::changed( false );
+}
+
+
 void LayoutConfig::updateStickyLimit()
 {
     int layoutsCnt = widget->listLayoutsDst->childCount();
-    widget->spinStickyDepth->setMaxValue( (layoutsCnt<=2) ? 2 : layoutsCnt - 1);
+	int maxDepth = layoutsCnt - 1;
+	
+	if( maxDepth < 2 ) {
+		maxDepth = 2;
+	}
+	
+	widget->spinStickyDepth->setMaxValue(maxDepth);
+/*	if( value > maxDepth )
+		setValue(maxDepth);*/
 }
 
 void LayoutConfig::add()
 {
     QListViewItem* sel = widget->listLayoutsSrc->selectedItem();
     if( sel == 0 )
-	return;
-    
-    widget->listLayoutsSrc->takeItem(sel);
-    widget->listLayoutsDst->insertItem(sel);
-    if( widget->listLayoutsDst->childCount() > 1 )
-	sel->moveItem(widget->listLayoutsDst->lastItem());
-// disabling temporary: does not work reliable in Qt :(
-//    widget->listLayoutsSrc->setSelected(sel, true);
-//    layoutSelChanged(sel);
+		return;
 
+    // Create a copy of the sel widget, as one might add the same layout more
+    // than one time, with different variants.
+    QListViewItem* toadd = copyLVI(sel, widget->listLayoutsDst);
+
+    widget->listLayoutsDst->insertItem(toadd);
+    if( widget->listLayoutsDst->childCount() > 1 )
+		toadd->moveItem(widget->listLayoutsDst->lastItem());
+// disabling temporary: does not work reliable in Qt :(
+//    widget->listLayoutsDst->setSelected(sel, true);
+//    layoutSelChanged(sel);
+	
     updateStickyLimit();
     changed();
 }
 
-void LayoutConfig::remove()
+void LayoutConfig::remove() 
 {
     QListViewItem* sel = widget->listLayoutsDst->selectedItem();
     QListViewItem* newSel = 0;
 
     if( sel == 0 )
-	return;
+        return;
 
     if( sel->itemBelow() )
-	newSel = sel->itemBelow();
+        newSel = sel->itemBelow();
     else
-    if( sel->itemAbove() )
-	newSel = sel->itemAbove();
+        if( sel->itemAbove() )
+            newSel = sel->itemAbove();
 
-    widget->listLayoutsDst->takeItem(sel);
-    widget->listLayoutsSrc->insertItem(sel);
+    delete sel;
     if( newSel )
-	widget->listLayoutsSrc->setSelected(newSel, true);
+        widget->listLayoutsSrc->setSelected(newSel, true);
     layoutSelChanged(newSel);
 
     updateStickyLimit();
@@ -192,15 +379,15 @@ void LayoutConfig::moveUp()
 {
     QListViewItem* sel = widget->listLayoutsDst->selectedItem();
     if( sel == 0 || sel->itemAbove() == 0 )
-	return;
+		return;
 
     if( sel->itemAbove()->itemAbove() == 0 ) {
-	widget->listLayoutsDst->takeItem(sel);
-	widget->listLayoutsDst->insertItem(sel);
-	widget->listLayoutsDst->setSelected(sel, true);
+		widget->listLayoutsDst->takeItem(sel);
+		widget->listLayoutsDst->insertItem(sel);
+		widget->listLayoutsDst->setSelected(sel, true);
     }
     else
-	sel->moveItem(sel->itemAbove()->itemAbove());
+		sel->moveItem(sel->itemAbove()->itemAbove());
 }
 
 void LayoutConfig::moveDown()
@@ -215,14 +402,24 @@ void LayoutConfig::moveDown()
 void LayoutConfig::variantChanged()
 {
     QListViewItem* selLayout = widget->listLayoutsDst->selectedItem();
-    if( !selLayout ) {
+    if( selLayout == NULL ) {
       widget->comboVariant->clear();
       widget->comboVariant->setEnabled(false);
       return;
     }
 
-    QString kbdLayout = lookupLocalized( m_rules->layouts(), selLayout->text(1) );
-    m_variants.replace(kbdLayout, widget->comboVariant->currentText().latin1());
+	QString selectedVariant = widget->comboVariant->currentText();
+	if( selectedVariant == DEFAULT_VARIANT_NAME )
+		selectedVariant = "";
+	selLayout->setText(LAYOUT_COLUMN_VARIANT, selectedVariant);
+}
+
+// helper
+static LayoutUnit getLayoutUnit(QListViewItem *sel)
+{
+	QString kbdLayout = sel->text(LAYOUT_COLUMN_MAP);
+	QString kbdVariant = sel->text(LAYOUT_COLUMN_VARIANT);
+	return LayoutUnit(kbdLayout, kbdVariant);
 }
 
 void LayoutConfig::latinChanged()
@@ -234,11 +431,15 @@ void LayoutConfig::latinChanged()
       return;
     }
 
-    QString kbdLayout = lookupLocalized(  m_rules->layouts(), selLayout->text( 1 ) );
-    if (  widget->chkLatin->isChecked() )
-        m_includes.replace( kbdLayout, "us" );
+	QString include;
+	if( widget->chkLatin->isChecked() )
+		include = "us";
     else
-        m_includes.replace( kbdLayout, "" );
+		include = "";
+	selLayout->setText(LAYOUT_COLUMN_INCLUDE, include);
+
+ 	LayoutUnit layoutUnitKey = getLayoutUnit(selLayout);
+	kdDebug() << "layout " << layoutUnitKey.toPair() << ", inc: " << include << endl;
 }
 
 void LayoutConfig::layoutSelChanged(QListViewItem *sel)
@@ -253,16 +454,18 @@ void LayoutConfig::layoutSelChanged(QListViewItem *sel)
         return;
     }
 
-    QString kbdLayout = lookupLocalized( m_rules->layouts(), sel->text(1) );
 
-// need better algorithm here for determining if needs us group
+	LayoutUnit layoutUnitKey = getLayoutUnit(sel);
+	QString kbdLayout = layoutUnitKey.layout;
+
+	// TODO: need better algorithm here for determining if needs us group
     if (  ! m_rules->isSingleGroup(kbdLayout) 
-	    || kbdLayout.startsWith("us") || kbdLayout.startsWith("en") ) {
+	    		|| kbdLayout.startsWith("us") || kbdLayout.startsWith("en") ) {
         widget->chkLatin->setEnabled( false );
     }
     else {
-        char* inc = m_includes[ kbdLayout ];
-        if ( inc && (strncmp(inc, "us", 2)==0 || strncmp(inc, "en", 2)==0) ) {
+		QString inc = sel->text(LAYOUT_COLUMN_INCLUDE);
+		if ( inc.startsWith("us") || inc.startsWith("en") ) {
             widget->chkLatin->setChecked(true);
         }
         else {
@@ -270,24 +473,21 @@ void LayoutConfig::layoutSelChanged(QListViewItem *sel)
         }
     }
 
-    QStringList vars = m_rules->getVariants(kbdLayout);
-
-    if( vars.count() == 0 ) {	// cowardly running away
-        updateLayoutCommand();
-        return;
-    }
-
-    char* variant = m_variants[kbdLayout];
-    widget->comboVariant->insertStringList(vars);
-
-    if( variant ) {
-      widget->comboVariant->setCurrentText(variant);
-    }
-    else {
-      widget->comboVariant->setCurrentItem(0);
-      m_variants.insert(kbdLayout, widget->comboVariant->currentText().latin1());
-    }
-
+	QStringList vars = m_rules->getAvailableVariants(kbdLayout);
+	kdDebug() << "layout " << kbdLayout << " has " << vars.count() << " variants" << endl;
+    
+	if( vars.count() > 0 ) {
+		vars.prepend(DEFAULT_VARIANT_NAME);
+		widget->comboVariant->insertStringList(vars);
+	
+		QString variant = sel->text(LAYOUT_COLUMN_VARIANT);
+		if( variant != NULL && variant.isEmpty() == false ) {
+			widget->comboVariant->setCurrentText(variant);
+		}
+		else {
+			widget->comboVariant->setCurrentItem(0);
+		}
+	}
     updateLayoutCommand();
 }
 
@@ -381,9 +581,10 @@ void LayoutConfig::updateLayoutCommand()
   QListViewItem* sel = widget->listLayoutsDst->selectedItem();
 
   if( sel ) {
-    QString kbdLayout = lookupLocalized(m_rules->layouts(), sel->text(1));
-//    char* variant = m_variants[kbdLayout];
+    QString kbdLayout = sel->text(LAYOUT_COLUMN_MAP);
     QString variant = widget->comboVariant->currentText();
+	if( variant == DEFAULT_VARIANT_NAME )
+		variant = "";
 
     setxkbmap = "setxkbmap"; //-rules " + m_rule
     setxkbmap += " -model " + lookupLocalized(m_rules->models(), widget->comboModel->currentText())
@@ -409,163 +610,44 @@ void LayoutConfig::changed()
 }
 
 
-void LayoutConfig::load()
+void LayoutConfig::loadRules()
 {
-  // open the config file
-  KConfig *config = new KConfig("kxkbrc", true);
-  config->setGroup("Layout");
+    // do we need this ?
+    // this could obly be used if rules are changed and 'Defaults' is pressed
+    delete m_rules;
+    m_rules = new XkbRules();
 
-  bool use = config->readBoolEntry( "Use", false );
+    QStringList modelsList;
+    QDictIterator<char> it(m_rules->models());
+    while (it.current()) {
+        modelsList.append(i18n(it.current()));
+        ++it;
+    }
+    modelsList.sort();
+	
+	widget->comboModel->clear();
+	widget->comboModel->insertStringList(modelsList);
+	widget->comboModel->setCurrentItem(0);
 
-  // find out which rule applies
-  //QString rule = "xfree86"; //config->readEntry("Rule", "xfree86");
-
-  // update other files
-  ruleChanged();
-
-  // find out about the model
-  QString model = config->readEntry("Model", "pc104");
-
-  // TODO: When no model and no layout are known (1st start), we really
-  // should ask the X-server about it's settings!
-
-  QString m_name = m_rules->models()[model];
-  widget->comboModel->setCurrentText(i18n(m_name.local8Bit()));
-
-  QString layout = config->readEntry("Layout", "us");
-  QString l_name = m_rules->layouts()[layout];
-
-  QStringList otherLayouts = config->readListEntry("Additional");
-  if( !l_name.isEmpty() ) {
-    otherLayouts.prepend(layout);
-  }
-// to optimize we should have gone from it.end to it.begin
-  for ( QStringList::Iterator it = otherLayouts.begin(); it != otherLayouts.end(); ++it ) {
-    QListViewItemIterator src_it( widget->listLayoutsSrc );
-    for ( ; src_it.current(); ++src_it ) {
-	QListViewItem* srcItem = src_it.current();
-	if ( *it == lookupLocalized(m_rules->layouts(), src_it.current()->text(1)) ) {
-    	    widget->listLayoutsSrc->takeItem(srcItem);
-    	    widget->listLayoutsDst->insertItem(srcItem);
-	    srcItem->moveItem(widget->listLayoutsDst->lastItem());
-	    break;
+	// fill in the additional layouts
+	widget->listLayoutsSrc->clear();
+	widget->listLayoutsDst->clear();
+	QDictIterator<char> it2(m_rules->layouts());
+	
+	while (it2.current())
+	{
+		QString layout = it2.currentKey();
+		QString layoutName = it2.current();
+		QListViewItem *item = new QListViewItem(widget->listLayoutsSrc);
+		
+		item->setPixmap(LAYOUT_COLUMN_FLAG, LayoutIcon::getInstance().findPixmap(layout, true));
+		item->setText(LAYOUT_COLUMN_NAME, i18n(layoutName.latin1()));
+		item->setText(LAYOUT_COLUMN_MAP, layout);
+		++it2;
 	}
-    }
-  }
-
-// reading variants
-  QStringList vars = config->readListEntry("Variants");
-  m_rules->parseVariants(vars, m_variants);
-//  m_rules->parseVariants( vars, m_variants );
-
-  QStringList incs = config->readListEntry( "Includes" );
-  m_rules->parseVariants( incs, m_includes, false );
-
-  bool showSingle = config->readBoolEntry( "ShowSingle", false );
-  widget->chkShowSingle->setChecked(showSingle);
-
-  bool showFlag = config->readBoolEntry( "ShowFlag", true );
-  widget->chkShowFlag->setChecked(showFlag);
-
-  bool enableXkbOptions = config->readBoolEntry( "EnableXkbOptions", true );
-  widget->chkEnableOptions->setChecked( enableXkbOptions );
-  bool resetOld = config->readBoolEntry("ResetOldOptions", false);
-  widget->checkResetOld->setChecked(resetOld);
-  QStringList options = config->readListEntry("Options");
-
-  for (QStringList::Iterator it = options.begin(); it != options.end(); ++it)
-    {
-      QString option = *it;
-      QString optionKey = option.mid(0, option.find(':'));
-      QString optionName = m_rules->options()[option];
-      OptionListItem *item = m_optionGroups[optionKey];
-      if (item != NULL) {
-        OptionListItem *child = item->findChildItem( option );
-
-        if ( child )
-          child->setState( QCheckListItem::On );
-	    else
-	      kdDebug() << "load: Unknown option " << option << endl;
-      }
-      else {
-	    kdDebug() << "load: Unknown option group " << optionKey << endl;
-      }
-    }
-
-  QString swMode = config->readEntry("SwitchMode", "Global");
-  widget->grpSwitching->setButton(0);
-
-   for(int ii=1; ii<3; ii++)
-    if( swMode == switchModes[ii] )
-      widget->grpSwitching->setButton(ii);
-
-  bool stickySwitching = config->readBoolEntry("StickySwitching", false);
-  widget->chkEnableSticky->setChecked(stickySwitching);
-  widget->spinStickyDepth->setEnabled(stickySwitching);
-  widget->spinStickyDepth->setValue( config->readEntry("StickySwitchingDepth", "1").toInt() + 1);
-
-  updateStickyLimit();
-
-  delete config;
-
-  widget->chkEnable->setChecked( use );
-  widget->grpLayouts->setEnabled(use);
-  widget->grpSwitching->setEnabled(use);
-
-  updateOptionsCommand();
-  emit KCModule::changed( false );
-}
-
-void LayoutConfig::ruleChanged()
-{
-//  if( rule == m_rule )
-//    return;
-
-//  m_rule = rule;
-
-  QString model; //, layout;
-  if (m_rules)
-    {
-      model = lookupLocalized(m_rules->models(), widget->comboModel->currentText());
-//      layout = lookupLocalized(m_rules->layouts(), layoutCombo->currentText());
-    }
-
-  delete m_rules;
-  m_rules = new KeyRules();
-
-  QStringList tmp;
-  widget->comboModel->clear();
-  QDictIterator<char> it(m_rules->models());
-  while (it.current())
-    {
-      tmp.append(i18n(it.current()));
-      ++it;
-    }
-  tmp.sort();
-  widget->comboModel->insertStringList(tmp);
-
-
-  // fill in the additional layouts  -- moved from load() by A. Rysin
-  widget->listLayoutsSrc->clear();
-  widget->listLayoutsDst->clear();
-  QDictIterator<char> it2(m_rules->layouts());
-  while (it2.current())
-    {
-      QCheckListItem *item = new QCheckListItem(widget->listLayoutsSrc, "");
-      QString addLayout = it2.currentKey();
-      item->setPixmap(0, LayoutIcon::findPixmap(addLayout, true));
-      item->setText(1, i18n(it2.current()) );
-      item->setText(2, "(" + addLayout + ")" );
-      ++it2;
-    }
-  widget->listLayoutsSrc->setSorting(1);	// from Qt3 QListView sorts by language
-
-
-  if (!model.isEmpty())
-  {
-    QString m_name = m_rules->models()[model];
-    widget->comboModel->setCurrentText(m_name);
-  }
+	widget->listLayoutsSrc->setSorting(LAYOUT_COLUMN_NAME);	// from Qt3 QListView sorts by language
+	
+	//TODO: reset options and xkb options
 }
 
 
@@ -606,176 +688,81 @@ QString LayoutConfig::createOptionString()
   return options;
 }
 
-void LayoutConfig::save()
-{
-  KConfig *config = new KConfig("kxkbrc", false);
-  config->setGroup("Layout");
-  //  config->writeEntry("Rule", ruleCombo->currentText());
-
-  QString model = lookupLocalized(m_rules->models(), widget->comboModel->currentText());
-  config->writeEntry("Model", model);
-
-  config->writeEntry("EnableXkbOptions", widget->chkEnableOptions->isChecked() );
-  config->writeEntry("ResetOldOptions", widget->checkResetOld->isChecked());
-  config->writeEntry("Options", createOptionString() );
-
-  QString layout;
-  QStringList otherLayouts;
-  QListViewItem *item = widget->listLayoutsDst->firstChild();
-  if( item ) {
-    layout = lookupLocalized(m_rules->layouts(), item->text(1));
-    if( !layout.isEmpty() )
-	config->writeEntry("Layout", layout);
-    item = item->nextSibling();
-  }
-  else {
-    widget->chkEnable->setChecked(false);
-  }
-
-  while (item) {
-	QString layout = lookupLocalized(m_rules->layouts(), item->text(1));
-	otherLayouts.append(layout);
-      item = item->nextSibling();
-  }
-  config->writeEntry("Additional", otherLayouts);
-
-  QStringList varList;
-  QStringList incList;
-  item = widget->listLayoutsDst->firstChild();
-  while ( item ) {
-    QString layout = lookupLocalized( m_rules->layouts(), item->text( 1 ) );
-    if( m_includes[ layout ] && m_includes[ layout ][ 0 ] != '\0' )
-    {
-        QString inc = layout;
-        inc += "(";
-        inc += m_includes[ layout ];
-        inc += ")";
-        incList.append(  inc );
-    }
-    if( m_variants[ layout ] && m_variants[ layout ][ 0 ] != '\0' )
-    {
-	QString var = layout;
-	var += "(";
-	var += m_variants[ layout ];
-	var += ")";
-	varList.append( var );
-    }
-
-    item = item->nextSibling();
-  }
-  config->writeEntry( "Includes", incList );
-  config->writeEntry("Variants", varList);
-
-  config->writeEntry("Use", widget->chkEnable->isChecked());
-  config->writeEntry("ShowSingle", widget->chkShowSingle->isChecked());
-  config->writeEntry("ShowFlag", widget->chkShowFlag->isChecked());
-
-  int modeId = widget->grpSwitching->id(widget->grpSwitching->selected());
-  if( modeId < 1 || modeId > 2 )
-    modeId = 0;
-
-  config->writeEntry("SwitchMode", switchModes[modeId]);
-
-  config->writeEntry("StickySwitching", widget->chkEnableSticky->isChecked());
-  config->writeEntry("StickySwitchingDepth", widget->spinStickyDepth->value() - 1);
-
-  config->sync();
-
-  delete config;
-
-  kapp->kdeinitExec("kxkb");
-  emit KCModule::changed( false );
-}
-
 
 void LayoutConfig::defaults()
 {
-  widget->chkEnable->setChecked(false);
-  ruleChanged();
+	loadRules();
+	m_kxkbConfig.setDefaults();
 
-  widget->comboModel->setCurrentText("pc104");
-  //layoutCombo->setCurrentText("us");
+	initUI();
 
-  widget->chkEnableOptions->setChecked( true );
-  widget->checkResetOld->setChecked( false );
-
-  widget->grpSwitching->setButton(0);
-
-  widget->chkEnableSticky->setChecked( false );
-  widget->spinStickyDepth->setEnabled( false );
-
-  QListViewItem *item = widget->listLayoutsSrc->firstChild();
-  while (item)
-    {
-      QCheckListItem *cli = dynamic_cast<QCheckListItem*>(item);
-      cli->setOn(false);
-      item = item->nextSibling();
-    }
-  emit KCModule::changed( true );
+	emit KCModule::changed( true );
 }
 
+
+OptionListItem::OptionListItem( OptionListItem *parent, const QString &text,
+								Type tt, const QString &optionName )
+	: QCheckListItem( parent, text, tt ), m_OptionName( optionName )
+{
+}
+
+OptionListItem::OptionListItem( QListView *parent, const QString &text,
+								Type tt, const QString &optionName )
+	: QCheckListItem( parent, text, tt ), m_OptionName( optionName )
+{
+}
+
+OptionListItem * OptionListItem::findChildItem( const QString& optionName )
+{
+	OptionListItem *child = static_cast<OptionListItem *>( firstChild() );
+
+	while ( child )
+	{
+		if ( child->optionName() == optionName )
+			break;
+		child = static_cast<OptionListItem *>( child->nextSibling() );
+	}
+
+	return child;
+}
+
+
 extern "C"
 {
-  KDE_EXPORT KCModule *create_keyboard_layout(QWidget *parent, const char *)
-  {
-    return new LayoutConfig(parent, "kcmlayout");
-  }
-
-  KDE_EXPORT KCModule *create_keyboard(QWidget *parent, const char *)
-  {
-    return new KeyboardConfig(parent, "kcmlayout");
-  }
-
-  KDE_EXPORT void init_keyboard()
-  {
-    KConfig *config = new KConfig("kcminputrc", true); // Read-only, no globals
-    config->setGroup("Keyboard");
-
-    XKeyboardState   kbd;
-    XKeyboardControl kbdc;
-
-    XGetKeyboardControl(kapp->getDisplay(), &kbd);
-    bool key = config->readBoolEntry("KeyboardRepeating", true);
-    kbdc.key_click_percent = config->readNumEntry("ClickVolume", kbd.key_click_percent);
-    kbdc.auto_repeat_mode = (key ? AutoRepeatModeOn : AutoRepeatModeOff);
-
-    XChangeKeyboardControl(kapp->getDisplay(),
-                           KBKeyClickPercent | KBAutoRepeatMode,
-                           &kbdc);
-
-    if( key ) {
-        int delay_ = config->readNumEntry("RepeatDelay", 250);
-        double rate_ = config->readDoubleNumEntry("RepeatRate", 30);
-        set_repeatrate(delay_, rate_);
-    }
-
-
-    int numlockState = config->readNumEntry( "NumLock", 2 );
-    if( numlockState != 2 )
-        numlockx_change_numlock_state( numlockState == 0 );
-
-    delete config;
-
-    config = new KConfig("kxkbrc", true, false);
-    config->setGroup("Layout");
-
-// Even if the layouts have been disabled we still want to set Xkb options
-// user can always switch them off now in the "Options" tab
-    bool enableXkbOptions = config->readBoolEntry("EnableXkbOptions", true);
-    if( enableXkbOptions ) {
-	bool resetOldOptions = config->readBoolEntry("ResetOldOptions", false);
-	QString options = config->readEntry("Options", "");
-	if( !XKBExtension::setXkbOptions(options, resetOldOptions) ) {
-	    kdDebug() << "Setting XKB options failed!" << endl;
+	KDE_EXPORT KCModule *create_keyboard_layout(QWidget *parent, const char *)
+	{
+		return new LayoutConfig(parent, "kcmlayout");
+	}
+	
+	KDE_EXPORT KCModule *create_keyboard(QWidget *parent, const char *)
+	{
+		return new KeyboardConfig(parent, "kcmlayout");
+	}
+	
+	KDE_EXPORT void init_keyboard()
+	{
+		KeyboardConfig::init_keyboard();
+		
+		KxkbConfig m_kxkbConfig;
+		m_kxkbConfig.load(KxkbConfig::LOAD_INIT_OPTIONS);
+	
+		if( m_kxkbConfig.m_useKxkb == true ) {
+			kapp->startServiceByDesktopName("kxkb");
+		}
+		else {
+		// Even if the layouts have been disabled we still want to set Xkb options
+		// user can always switch them off now in the "Options" tab
+			if( m_kxkbConfig.m_enableXkbOptions ) {
+				if( !XKBExtension::setXkbOptions(m_kxkbConfig.m_options, m_kxkbConfig.m_resetOldOptions) ) {
+					kdDebug() << "Setting XKB options failed!" << endl;
+				}
+			}
+		}
 	}
-    }
-
-    if (config->readBoolEntry("Use", false) == true)
-        kapp->startServiceByDesktopName("kxkb");
-    delete config;
-  }
 }
 
+
+
 #if 0// do not remove!
 // please don't change/fix messages below
 // they're taken from XFree86 as is and should stay the same
